"""
借数凑整法加法实现
通过"借"和"还"的方式将数字调整为整数，简化计算
如98+67可以看作(100-2)+67=100+67-2=167-2=165
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class BorrowRoundingAddition(MathCalculator):
    """借数凑整法加法算法"""
    
    def __init__(self):
        super().__init__("借数凑整法", "通过借数凑整简化计算，先借后还", priority=4)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：其中一个数接近整十、整百等"""
        if formula.type != "addition":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            addend1, addend2 = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(addend1, (int, float)) and isinstance(addend2, (int, float))):
                return False

            # 转换为整数
            addend1, addend2 = int(addend1), int(addend2)
            
            # 只处理正数且大于10
            if addend1 <= 10 or addend2 <= 10:
                return False
            
            ## 只处理两数总和大于50的情况
            if addend1 + addend2 <= 50:
                return False
            
            # 检查是否有数字接近整十、整百等（差距在1-9之间）
            def is_near_round(n):
                # 检查接近整百
                if n % 100 != 0 and n >= 50:
                    gap_to_hundred = 100 - (n % 100)

                    if 1 <= gap_to_hundred <= 20:
                        return True, gap_to_hundred, (n // 100 + 1) * 100
                    
                # 检查接近整十
                if n % 10 != 0:
                    gap_to_ten = 10 - (n % 10)
                    if 1 <= gap_to_ten <= 9:
                        return True, gap_to_ten, (n // 10 + 1) * 10
                
                return False, 0, n
            
            # 至少有一个数接近整数
            near1, gap1, round1 = is_near_round(addend1)
            near2, gap2, round2 = is_near_round(addend2)

            # 限制一下，两个数的gap相等时，刚好可以互补
            if (near1 or near2) and (gap1 + gap2) == 10:
                return True
            return False
        except Exception as e:
            print('借数凑整法加法算法异常', e)
            return False
    
    def _analyze_borrowing(self, addend1: int, addend2: int) -> tuple:
        """分析借数策略"""
        def get_borrowing_info(n):
            # 检查接近整百
            if n % 100 != 0 and n >= 50:
                gap_to_hundred = 100 - (n % 100)
                if 1 <= gap_to_hundred <= 20:  # 只考虑较小的借数
                    return True, gap_to_hundred, (n // 100 + 1) * 100, "整百"
                
            # 检查接近整十
            if n % 10 != 0:
                gap_to_ten = 10 - (n % 10)
                if 1 <= gap_to_ten <= 5:  # 只考虑较小的借数
                    return True, gap_to_ten, (n // 10 + 1) * 10, "整十"
            
            return False, 0, n, "无"
        
        # 分析两个数
        can_borrow1, gap1, round1, type1 = get_borrowing_info(addend1)
        can_borrow2, gap2, round2, type2 = get_borrowing_info(addend2)
        
        # 选择借数较少的数进行借数
        if can_borrow1 and can_borrow2:
            if gap1 >= gap2:
                return addend1, addend2, gap1, round1, type1, True
            else:
                return addend2, addend1, gap2, round2, type2, False
        elif can_borrow1:
            return addend1, addend2, gap1, round1, type1, True
        elif can_borrow2:
            return addend2, addend1, gap2, round2, type2, False
        else:
            return addend1, addend2, 0, addend1, "无", True
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建借数凑整法步骤"""
        numbers = formula.get_numbers()
        addend1 = int(numbers[0].get_numeric_value())
        addend2 = int(numbers[1].get_numeric_value())
        
        # 分析借数策略
        borrow_num, other_num, gap, round_num, round_type, first_is_borrow = self._analyze_borrowing(addend1, addend2)
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{addend1} + {addend2} 使用借数凑整法",
            operation="识别借数凑整法",
            result=f"{borrow_num} 接近{round_type} {round_num}，借 {gap} 凑整"
        ))
        
        steps.append(CalculationStep(
            description=f"借数凑整：{borrow_num} + {gap} = {round_num}",
            operation="借数凑整",
            result=f"将 {borrow_num} 看作 {round_num}"
        ))
        
        # 用整数计算
        temp_sum = round_num + other_num
        steps.append(CalculationStep(
            description=f"用整数计算：{round_num} + {other_num} = {temp_sum}",
            operation="整数计算",
            result=temp_sum
        ))
        
        # 还数
        final_result = temp_sum - gap
        steps.append(CalculationStep(
            description=f"还回借数：{temp_sum} - {gap} = {final_result}",
            operation="还数",
            result=final_result,
            formula=f"借数凑整法：借{gap}凑{round_type}，计算后还{gap}"
        ))
        
        return steps


"""
立方体积单位换算算子
"""

from typing import List, Dict
from ..base_types import UnitCalculator, UnitProblem, UnitType, UnitValue, ConversionStep

class VolumeUnitCalculator(UnitCalculator):
    """立方体积单位换算算子"""
    def __init__(self):
        super().__init__("立方体积单位换算", "处理立方米、立方分米、立方厘米、升、毫升之间的换算", UnitType.VOLUME)

    def _initialize_conversion_rules(self) -> Dict[str, Dict[str, float]]:
        # 初始化立方体积单位换算规则，支持所有常见英文/中文表达
        return {
        # 立方米
        "m³": {"m³": 1, "m³": 1, "dm³": 1000, "dm³": 1000, "cm³": 1000000, "cm³": 1000000, "L": 1000, "mL": 1000000,
            "立方米": 1, "立方分米": 1000, "立方厘米": 1000000, "升": 1000, "毫升": 1000000},
        "m³": {"m³": 1, "m³": 1, "dm³": 1000, "dm³": 1000, "cm³": 1000000, "cm³": 1000000, "L": 1000, "mL": 1000000,
            "立方米": 1, "立方分米": 1000, "立方厘米": 1000000, "升": 1000, "毫升": 1000000},
        "立方米": {"m³": 1, "m³": 1, "dm³": 1000, "dm³": 1000, "cm³": 1000000, "cm³": 1000000, "L": 1000, "mL": 1000000,
              "立方米": 1, "立方分米": 1000, "立方厘米": 1000000, "升": 1000, "毫升": 1000000},
        # 立方分米
        "dm³": {"m³": 0.001, "m³": 0.001, "dm³": 1, "dm³": 1, "cm³": 1000, "cm³": 1000, "L": 1, "mL": 1000,
             "立方米": 0.001, "立方分米": 1, "立方厘米": 1000, "升": 1, "毫升": 1000},
        "dm³": {"m³": 0.001, "m³": 0.001, "dm³": 1, "dm³": 1, "cm³": 1000, "cm³": 1000, "L": 1, "mL": 1000,
             "立方米": 0.001, "立方分米": 1, "立方厘米": 1000, "升": 1, "毫升": 1000},
        "立方分米": {"m³": 0.001, "m³": 0.001, "dm³": 1, "dm³": 1, "cm³": 1000, "cm³": 1000, "L": 1, "mL": 1000,
            "立方米": 0.001, "立方分米": 1, "立方厘米": 1000, "升": 1, "毫升": 1000},
        # 立方厘米
        "cm³": {"m³": 0.000001, "m³": 0.000001, "dm³": 0.001, "dm³": 0.001, "cm³": 1, "cm³": 1, "L": 0.001, "mL": 1,
             "立方米": 0.000001, "立方分米": 0.001, "立方厘米": 1, "升": 0.001, "毫升": 1},
        "cm³": {"m³": 0.000001, "m³": 0.000001, "dm³": 0.001, "dm³": 0.001, "cm³": 1, "cm³": 1, "L": 0.001, "mL": 1,
             "立方米": 0.000001, "立方分米": 0.001, "立方厘米": 1, "升": 0.001, "毫升": 1},
        "立方厘米": {"m³": 0.000001, "m³": 0.000001, "dm³": 0.001, "dm³": 0.001, "cm³": 1, "cm³": 1, "L": 0.001, "mL": 1,
            "立方米": 0.000001, "立方分米": 0.001, "立方厘米": 1, "升": 0.001, "毫升": 1},
        # 升
        "L": {"m³": 0.001, "m³": 0.001, "dm³": 1, "dm³": 1, "cm³": 1000, "cm³": 1000, "L": 1, "mL": 1000,
          "立方米": 0.001, "立方分米": 1, "立方厘米": 1000, "升": 1, "毫升": 1000},
        "升": {"m³": 0.001, "m³": 0.001, "dm³": 1, "dm³": 1, "cm³": 1000, "cm³": 1000, "L": 1, "mL": 1000,
           "立方米": 0.001, "立方分米": 1, "立方厘米": 1000, "升": 1, "毫升": 1000},
        # 毫升
        "mL": {"m³": 0.000001, "m³": 0.000001, "dm³": 0.001, "dm³": 0.001, "cm³": 1, "cm³": 1, "L": 0.001, "mL": 1,
            "立方米": 0.000001, "立方分米": 0.001, "立方厘米": 1, "升": 0.001, "毫升": 1},
        "毫升": {"m³": 0.000001, "m³": 0.000001, "dm³": 0.001, "dm³": 0.001, "cm³": 1, "cm³": 1, "L": 0.001, "mL": 1,
             "立方米": 0.000001, "立方分米": 0.001, "立方厘米": 1, "升": 0.001, "毫升": 1},
    }

    def is_match_pattern(self, problem: UnitProblem) -> bool:
        """判断是否为立方体积单位换算问题"""
        return problem.unit_type == UnitType.VOLUME

    def solve(self, problem: UnitProblem) -> tuple[float, List[ConversionStep]]:
        """求解立方体积单位换算问题"""
        steps = []
        total_result = 0
        operator = "+"
        for source_value in problem.source_values:
            conversion_factor = self.get_conversion_factor(source_value.unit, problem.target_unit)
            if conversion_factor is None:
                print(f"无法从{source_value.unit}换算到{problem.target_unit}")
                raise ValueError(f"无法从{source_value.unit}换算到{problem.target_unit}")
            converted_value = source_value.value * conversion_factor
            if operator == '-':
                total_result -= converted_value
            elif operator == '+':
                total_result += converted_value
            operator = source_value.operator
            formula = self._get_standard_formula(source_value.unit, problem.target_unit)
            step = ConversionStep(
                description=f"换算{source_value.unit}到{problem.target_unit}",
                operation="单位换算",
                from_value=source_value,
                result=UnitValue(converted_value, problem.target_unit),
                formula=formula
            )
            steps.append(step)
        if len(problem.source_values) > 1:
            formula_parts = []
            for v in problem.source_values:
                factor = self.get_conversion_factor(v.unit, problem.target_unit)
                if factor is None:
                    raise ValueError(f"无法从{v.unit}换算到{problem.target_unit}")
                formula_parts.append(str(v.value * factor) + v.operator)
            formula = ' '.join(formula_parts).rstrip('+')
            step = ConversionStep(
                description=f"合并所有换算结果",
                operation="求和",
                from_value=UnitValue(0, problem.target_unit),
                result=UnitValue(total_result, problem.target_unit),
                formula=f"总和 = {formula}"
            )
            steps.append(step)
        return total_result, steps

    def _get_standard_formula(self, from_unit: str, to_unit: str) -> str:
        """获取标准化的换算公式（大单位 = 小单位倍数）"""
        unit_hierarchy = {
            "m³": 5, "立方米": 5,
            "dm³": 4, "立方分米": 4, "L": 4, "升": 4,
            "cm³": 3, "立方厘米": 3, "mL": 3, "毫升": 3
        }
        from_level = unit_hierarchy.get(from_unit, 0)
        to_level = unit_hierarchy.get(to_unit, 0)
        factor = self.get_conversion_factor(from_unit, to_unit)
        if factor is None or factor == 0:
            return f"1{from_unit} = 1{to_unit}"
        if from_level > to_level:
            big_unit, small_unit = from_unit, to_unit
        else:
            big_unit, small_unit = to_unit, from_unit
            factor = 1 / factor if factor else 1
        if factor and factor != 1:
            if factor == int(factor):
                return f"1{big_unit} = {int(factor)}{small_unit}"
            else:
                return f"1{big_unit} = {factor}{small_unit}"
        else:
            return f"1{from_unit} = 1{to_unit}"



"""
乘5速算法实现
任何数乘5，先乘10再除以2
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class MultiplyByFive(MathCalculator):
    """乘5速算法"""
    
    def __init__(self):
        super().__init__("乘5速算", "任何数乘5，先乘10再除以2", priority=4)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：任何数乘以5"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            return (a == 5 and isinstance(b, (int, float))) or (b == 5 and isinstance(a, (int, float)))
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建乘5步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 确定哪个是5
        if a == 5:
            num = b
        else:
            num = a
        
        # 计算
        step1_result = num * 10
        final_result = step1_result / 2
        
        steps = [
            CalculationStep(
                description=f"{num} × 5 使用乘5速算法",
                operation="识别模式",
                result="使用先乘10再除以2的方法"
            ),
            CalculationStep(
                description=f"{num} × 10 = {step1_result}",
                operation="乘以10",
                result=step1_result
            ),
            CalculationStep(
                description=f"{step1_result} ÷ 2 = {final_result}",
                operation="除以2", 
                result=int(final_result) if final_result.is_integer() else final_result,
                formula="数×5 = (数×10)÷2"
            )
        ]
        
        return steps


"""
同头数减法实现
处理十位相同的两位数减法，如74-42=32
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class SameHeadSubtraction(MathCalculator):
    """同头数减法算法"""
    
    def __init__(self):
        super().__init__("同头数减法", "十位相同的两位数减法", priority=8)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：十位相同的两位数减法"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            minuend, subtrahend = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(minuend, int) and isinstance(subtrahend, int)):
                return False
            
            # 两位数且十位相同，个位能够减
            if not (10 <= minuend <= 99 and 10 <= subtrahend <= 99):
                return False
            
            minuend_tens = minuend // 10
            subtrahend_tens = subtrahend // 10
            minuend_ones = minuend % 10
            subtrahend_ones = subtrahend % 10
            
            # 十位相同且个位能够减（不需要借位）
            return (minuend_tens == subtrahend_tens and minuend_ones >= subtrahend_ones)
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建同头数减法步骤"""
        numbers = formula.get_numbers()
        minuend = int(numbers[0].get_numeric_value())
        subtrahend = int(numbers[1].get_numeric_value())
        
        minuend_tens = minuend // 10
        minuend_ones = minuend % 10
        subtrahend_tens = subtrahend // 10
        subtrahend_ones = subtrahend % 10
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 识别同头数减法",
            operation="识别同头数",
            result=f"十位都是{minuend_tens}，使用同头数减法"
        ))
        
        steps.append(CalculationStep(
            description=f"分解：{minuend} = {minuend_tens}0 + {minuend_ones}, {subtrahend} = {subtrahend_tens}0 + {subtrahend_ones}",
            operation="数字分解",
            result=f"十位相同：{minuend_tens}，个位：{minuend_ones}-{subtrahend_ones}"
        ))
        
        # 十位相减（同头数相减为0）
        tens_result = 0
        ones_result = minuend_ones - subtrahend_ones
        
        steps.append(CalculationStep(
            description=f"十位相减：{minuend_tens} - {subtrahend_tens} = 0",
            operation="同头数相减",
            result=0
        ))
        
        steps.append(CalculationStep(
            description=f"个位相减：{minuend_ones} - {subtrahend_ones} = {ones_result}",
            operation="个位相减",
            result=ones_result
        ))
        
        final_result = tens_result * 10 + ones_result
        steps.append(CalculationStep(
            description=f"组合结果：0 + {ones_result} = {final_result}",
            operation="合并结果",
            result=final_result,
            formula="同头数减法：十位相同时，直接计算个位差"
        ))
        
        return steps


"""
九九表速算除法实现
利用乘法表反推除法
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class MultiplicationTableDivision(MathCalculator):
    """九九表速算除法算法"""
    
    def __init__(self):
        super().__init__("九九表速算", "熟表反推，除数快算", priority=4)
    
    def _multiplication_table(self) -> dict:
        """生成九九乘法表"""
        table = {}
        for i in range(1, 10):
            for j in range(1, 10):
                table[i * j] = (i, j)
        return table
    
    def _extended_table(self) -> dict:
        """生成扩展乘法表（包括12以内的乘法）"""
        table = {}
        for i in range(1, 13):
            for j in range(1, 13):
                product = i * j
                if product not in table:
                    table[product] = []
                table[product].append((i, j))
        return table
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：能在九九表或扩展表中找到的除法"""
        if formula.type != "division":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            dividend, divisor = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(dividend, int) and isinstance(divisor, int)):
                return False
            
            if divisor == 0 or divisor == 1:  # 排除除数为1的情况
                return False
            
            # 除数必须在2-12范围内（常见乘法表范围）
            if divisor < 2 or divisor > 12:
                return False
            
            # 被除数必须能整除除数
            if dividend % divisor != 0:
                return False
            
            quotient = dividend // divisor
            
            # 商也必须在1-12范围内
            if quotient < 1 or quotient > 12:
                return False
            
            # 被除数应该在扩展乘法表范围内（最大144）
            return dividend <= 144
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建九九表速算除法步骤"""
        numbers = formula.get_numbers()
        dividend, divisor = [elem.get_numeric_value() for elem in numbers]
        
        # 确保是整数
        dividend_int = int(dividend)
        divisor_int = int(divisor)
        
        quotient = dividend_int // divisor_int
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"识别九九表速算：{dividend_int} ÷ {divisor_int}",
            operation="识别模式",
            result="熟表反推，除数快算"
        ))
        
        # 判断使用哪个乘法表
        if dividend_int <= 81 and divisor_int <= 9 and quotient <= 9:
            table_type = "九九乘法表"
        else:
            table_type = "扩展乘法表"
        
        steps.append(CalculationStep(
            description=f"使用{table_type}反推：{divisor_int} × ? = {dividend_int}",
            operation="反推思路",
            result=f"寻找 {divisor_int} 的倍数等于 {dividend_int}"
        ))
        
        # 展示思维过程
        if divisor_int <= 9:
            # 展示该数在乘法表中的倍数
            multiples = []
            for i in range(1, min(10, dividend_int // divisor_int + 2)):
                multiple = divisor_int * i
                if multiple == dividend_int:
                    multiples.append(f"{divisor_int}×{i}={multiple} ✓")
                elif multiple <= dividend_int + divisor_int:
                    multiples.append(f"{divisor_int}×{i}={multiple}")
            
            steps.append(CalculationStep(
                description=f"{divisor_int}的倍数：{' '.join(multiples[:4])}",
                operation="列举倍数",
                result=f"找到：{divisor_int} × {quotient} = {dividend_int}"
            ))
        
        # 直接给出答案
        steps.append(CalculationStep(
            description=f"根据乘法表：{divisor_int} × {quotient} = {dividend_int}",
            operation="表查结果",
            result=quotient
        ))
        
        steps.append(CalculationStep(
            description=f"因此：{dividend_int} ÷ {divisor_int} = {quotient}",
            operation="得出答案",
            result=quotient,
            formula="九九表反推：a × b = c，则 c ÷ a = b"
        ))
        
        # 如果是经典九九表范围，添加口诀提示
        if dividend_int <= 81 and divisor_int <= 9 and quotient <= 9:
            if quotient <= divisor_int:
                steps.append(CalculationStep(
                    description=f"乘法口诀：{quotient}×{divisor_int}={dividend_int}",
                    operation="口诀验证",
                    result=f"背诵口诀帮助快速计算"
                ))
            else:
                steps.append(CalculationStep(
                    description=f"乘法口诀：{divisor_int}×{quotient}={dividend_int}",
                    operation="口诀验证",
                    result=f"背诵口诀帮助快速计算"
                ))
        
        return steps


"""
小数倍数乘法算子
实现×10、×100、×0.1、×0.01的小数点移动规律
"""

from typing import List
from .decima_calculator import DecimaCalculator
from ..base_types import Formula, CalculationStep


class DecimalScaleMultiply(DecimaCalculator):
    """小数倍数乘法算子"""
    
    def __init__(self):
        super().__init__("小数点移动乘法", "乘10、100、0.1、0.01规律", priority=8)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：小数乘法且其中一个因数是10的幂或其倒数"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            
            # 检查是否有一个数是10的幂或其倒数
            scale_factors = [10, 100, 1000, 0.1, 0.01, 0.001]
            return a in scale_factors or b in scale_factors
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建倍数乘法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        steps = []
        
        # 确定哪个是倍数，哪个是普通数
        scale_factors = [10, 100, 1000, 0.1, 0.01, 0.001]
        if a in scale_factors:
            scale_factor, base_num = a, b
        else:
            scale_factor, base_num = b, a
        
        # 分析移动规律
        if scale_factor >= 1:
            # ×10, ×100等：右移小数点
            power = int(scale_factor).bit_length() - 1 if int(scale_factor) > 0 else 0
            if scale_factor == 10:
                power = 1
            elif scale_factor == 100:
                power = 2
            elif scale_factor == 1000:
                power = 3
            
            steps.append(CalculationStep(
                description=f"{base_num} × {scale_factor}，小数点右移{power}位",
                operation="分析移动规律",
                result=f"右移{power}位"
            ))
            
            final_result = base_num * scale_factor
            steps.append(CalculationStep(
                description=f"右移{power}位：{base_num} → {final_result}",
                operation="移动小数点",
                result=final_result,
                formula=f"×{scale_factor} = 小数点右移{power}位"
            ))
        else:
            # ×0.1, ×0.01等：左移小数点
            power = len(str(scale_factor).split('.')[1]) if '.' in str(scale_factor) else 0
            
            steps.append(CalculationStep(
                description=f"{base_num} × {scale_factor}，小数点左移{power}位",
                operation="分析移动规律",
                result=f"左移{power}位"
            ))
            
            final_result = base_num * scale_factor
            steps.append(CalculationStep(
                description=f"左移{power}位：{base_num} → {final_result}",
                operation="移动小数点",
                result=final_result,
                formula=f"×{scale_factor} = 小数点左移{power}位"
            ))
        
        return steps



"""
差额法(求差法)乘法实现
两个数与某个中间数等距时，积等于中间数的平方减去距离的平方
适用于两个乘数关于一个整数对称的情况
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class DifferenceMethodMultiplication(MathCalculator):
    """差额法(求差法)乘法算法"""
    
    def __init__(self):
        super().__init__("差额法", "两数关于中间数对称时，积=中间数平方-距离平方", priority=6)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两个乘数关于某个整数对称"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, (int, float)) and isinstance(b, (int, float))):
                return False
            
            # 转换为整数便于计算
            a, b = int(a), int(b)
            
            # 只处理正数且大于等于10的情况
            if a < 10 or b < 10:
                return False
            
            # 确保a <= b
            if a > b:
                a, b = b, a
            
            # 检查是否存在中间数使得两数对称
            # 中间数应该是 (a + b) / 2
            middle = (a + b) / 2
            
            # 中间数必须是整数或半整数
            if not (middle == int(middle) or middle == int(middle) + 0.5):
                return False
            
            # 中间数必需整十、整百的数
            if not (middle % 10 == 0 or middle % 100 == 0):
                return False
            
            # 计算距离
            distance = abs(b - middle)

            # 距离必需小于20
            if distance >= 20:
                return False
            # 距离应该相等且为正整数或半整数
            if abs(a - middle) != distance:
                return False
            
            # 距离不能太小（至少为1）且不能太大（保证有优化价值）
            return 1 <= distance <= 10
            
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建差额法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 转换为整数
        a, b = int(a), int(b)
        
        # 确保a <= b便于描述
        if a > b:
            a, b = b, a
        
        # 计算中间数和距离
        middle = (a + b) / 2
        distance = b - middle
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{a} × {b} 使用差额法",
            operation="识别差额法",
            result=f"两数关于中间数{middle}对称"
        ))
        
        steps.append(CalculationStep(
            description=f"计算中间数：({a} + {b}) ÷ 2 = {middle}",
            operation="计算中间数",
            result=middle
        ))
        
        steps.append(CalculationStep(
            description=f"计算距离：{b} - {middle} = {distance}，{middle} - {a} = {distance}",
            operation="计算距离",
            result=f"距离为{distance}"
        ))
        
        # 计算中间数的平方
        middle_square = middle ** 2
        steps.append(CalculationStep(
            description=f"中间数平方：{middle}² = {middle_square}",
            operation="计算中间数平方",
            result=middle_square
        ))
        
        # 计算距离的平方
        distance_square = distance ** 2
        steps.append(CalculationStep(
            description=f"距离平方：{distance}² = {distance_square}",
            operation="计算距离平方",
            result=distance_square
        ))
        
        # 计算最终结果
        final_result = middle_square - distance_square
        if final_result.is_integer():
            final_result = int(final_result)
        
        steps.append(CalculationStep(
            description=f"最终结果：{middle_square} - {distance_square} = {final_result}",
            operation="计算差值",
            result=final_result,
            formula="差额法：(m+d)(m-d) = m² - d²，其中m为中间数，d为距离"
        ))
        
        return steps


"""
乘以11实现
两位数及以上数乘11，两边一拉，中间相加，满十进一
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class MultiplyByEleven(MathCalculator):
    """乘以11算法"""
    
    def __init__(self):
        super().__init__("乘11速算", "两位数及以上乘11，两边一拉中间相加", priority=4)
        self.formula = "(10xa+b)x11=ax100+(a+b)x10+b"
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两位数及以上乘以11"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, int) and isinstance(b, int)):
                return False
            
            # 检查是否有一个数是11，另一个数是两位数及以上
            if a == 11 and abs(b) >= 10:
                return True
            elif b == 11 and abs(a) >= 10:
                return True
            
            return False
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建乘11速算步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 确定哪个是11，哪个是被乘数
        if a == 11:
            multiplied_num = abs(b)
            is_negative = b < 0
        else:
            multiplied_num = abs(a)
            is_negative = a < 0
        
        str_num = str(multiplied_num)
        digits = list(map(int, str_num))
        n = len(digits)
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{multiplied_num} × 11 使用乘11速算法",
            operation="识别模式",
            result="两边一拉，中间相加，满十进位"
        ))
        
        steps.append(CalculationStep(
            description=f"原数各位数字：{' '.join(str_num)}",
            operation="数字分解",
            result=f"从左到右：{', '.join(str_num)}"
        ))
        
        # 乘11速算法的核心：两边一拉，中间相加
        # 对于数字 abc，结果构建为 a|(a+b)|(b+c)|c
        
        raw_result = []
        addition_details = []
        
        # 第一位：保持不变
        raw_result.append(digits[0])
        addition_details.append(f"首位：{digits[0]}")
        
        # 中间各位：相邻数字相加
        for i in range(n - 1):
            sum_val = digits[i] + digits[i + 1]
            raw_result.append(sum_val)
            addition_details.append(f"{digits[i]} + {digits[i+1]} = {sum_val}")
        
        # 最后一位：保持不变
        raw_result.append(digits[-1])
        addition_details.append(f"末位：{digits[-1]}")
        
        steps.append(CalculationStep(
            description=f"两边一拉中间相加：{'; '.join(addition_details)}",
            operation="构建初步结果",
            result=f"得到：{' | '.join(map(str, raw_result))}"
        ))
        
        # 处理进位：从右向左
        final_digits = []
        carry = 0
        carry_process = []
        
        for i in range(len(raw_result) - 1, -1, -1):
            total = raw_result[i] + carry
            if total >= 10:
                carry = total // 10
                digit = total % 10
                final_digits.insert(0, digit)
                carry_process.append(f"{raw_result[i]}+{total-raw_result[i]}={total} → {digit}(进位{carry})")
            else:
                carry = 0
                digit = total
                final_digits.insert(0, digit)
                if raw_result[i] != total:
                    carry_process.append(f"{raw_result[i]}+{total-raw_result[i]}={total}")
        
        if carry > 0:
            final_digits.insert(0, carry)
        
        if any(x >= 10 for x in raw_result):
            steps.append(CalculationStep(
                description=f"处理进位：{'; '.join(reversed(carry_process))}",
                operation="进位调整",
                result="从右向左处理满十进位"
            ))
        
        # 构建最终结果
        final_result = int(''.join(map(str, final_digits)))
        if is_negative:
            final_result = -final_result
        
        steps.append(CalculationStep(
            description=f"最终结果：{final_result}",
            operation="完成计算",
            result=final_result,
            formula="乘11法则：两边一拉，中间相加，满十进一"
        ))
        
        return steps


"""
分数计算系统基础类型定义
"""

from abc import ABC, abstractmethod
from typing import List, Dict, Any, Optional, Union
from dataclasses import dataclass
from fractions import Fraction
from decimal import Decimal
from enum import Enum
import re


class FractionType(Enum):
    """分数类型枚举"""
    PROPER = "proper"           # 真分数
    IMPROPER = "improper"       # 假分数
    MIXED = "mixed"             # 带分数
    INTEGER = "integer"         # 整数
    DECIMAL = "decimal"         # 小数
    PERCENT = "percent"         # 百分数


class OperationType(Enum):
    """运算类型枚举"""
    ADD = "add"
    SUBTRACT = "subtract"
    MULTIPLY = "multiply"
    DIVIDE = "divide"
    COMPARE = "compare"
    CONVERT = "convert"
    RECIPROCAL = "reciprocal"
    REDUCE = "reduce"


@dataclass
class FractionOperand:
    """分数操作数"""
    raw: str                    # 原始文本
    fraction: Fraction          # 标准分数表示
    fraction_type: FractionType # 分数类型
    is_mixed: bool = False      # 是否为带分数
    whole_part: int = 0         # 带分数的整数部分
    original_format: str = ""   # 原始格式（LaTeX等）


@dataclass
class FractionProblem:
    """分数问题定义"""
    id: str
    original_text: str
    operation: OperationType
    operands: List[FractionOperand]
    target_format: str = "fraction"  # 期望输出格式
    precision: int = 4
    display_mixed: bool = False


@dataclass
class FractionStep:
    """分数计算步骤"""
    description: str
    operation: str
    inputs: List[str] = None
    result: Union[str, Fraction] = None
    formula: Optional[str] = None
    validation: bool = True
    meta: Dict[str, Any] = None
    def __str__(self):
        return f"{self.description}: {self.operation} → {self.result}"
    def to_dict(self) -> Dict[str, Any]:
        result = self.result
        return {
            "description": self.description,
            "operation": self.operation,
            "result": result,
            "formula": self.formula
        }


@dataclass
class FractionResult:
    """分数计算结果"""
    success: bool
    name: Optional[str] = None
    result: Union[Fraction, str, float] = None
    steps: List[FractionStep] = None
    step_count: int = 0
    formula: Optional[str] = None
    validation: bool = True
    error: Optional[str] = None


class FractionCalculator(ABC):
    """分数算子基类"""
    
    def __init__(self, name: str, description: str, priority: int = 5):
        self.name = name
        self.description = description
        self.priority = priority
    
    @abstractmethod
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """
        匹配模式判断
        返回: {"matched": bool, "score": float, "reason": str}
        """
        pass
    
    @abstractmethod
    def solve(self, problem: FractionProblem) -> FractionResult:
        """解决分数问题"""
        pass
    
    def validate_result(self, problem: FractionProblem, result: Union[Fraction, float]) -> bool:
        """验证结果正确性"""
        try:
            # 默认验证逻辑：重新计算对比
            return True
        except:
            return False


class FractionUtils:
    """分数工具类"""
    
    @staticmethod
    def parse_fraction_text(text: str) -> FractionOperand:
        """解析分数文本"""
        text = text.strip()
        
        # LaTeX格式: #frac{a}{b}#
        latex_match = re.match(r'#frac\{(\d+)\}\{(\d+)\}#', text)
        if latex_match:
            numer, denom = int(latex_match.group(1)), int(latex_match.group(2))
            return FractionOperand(
                raw=text,
                fraction=Fraction(numer, denom),
                fraction_type=FractionType.PROPER if numer < denom else FractionType.IMPROPER,
                original_format="latex"
            )
        
        # 带分数格式: #a frac{b}{c}# 或 a b/c
        mixed_latex = re.match(r'#(\d+)frac\{(\d+)\}\{(\d+)\}#', text)
        if mixed_latex:
            whole, numer, denom = int(mixed_latex.group(1)), int(mixed_latex.group(2)), int(mixed_latex.group(3))
            fraction = Fraction(whole * denom + numer, denom)
            return FractionOperand(
                raw=text,
                fraction=fraction,
                fraction_type=FractionType.MIXED,
                is_mixed=True,
                whole_part=whole,
                original_format="latex"
            )
        
        # 普通分数: a/b
        frac_match = re.match(r'(\d+)/(\d+)', text)
        if frac_match:
            numer, denom = int(frac_match.group(1)), int(frac_match.group(2))
            return FractionOperand(
                raw=text,
                fraction=Fraction(numer, denom),
                fraction_type=FractionType.PROPER if numer < denom else FractionType.IMPROPER
            )
        
        # 小数
        if '.' in text:
            try:
                decimal_val = float(text)
                return FractionOperand(
                    raw=text,
                    fraction=Fraction(decimal_val).limit_denominator(),
                    fraction_type=FractionType.DECIMAL
                )
            except ValueError:
                pass
        
        # 百分数
        if text.endswith('%'):
            try:
                percent_val = float(text[:-1]) / 100
                return FractionOperand(
                    raw=text,
                    fraction=Fraction(percent_val).limit_denominator(),
                    fraction_type=FractionType.PERCENT
                )
            except ValueError:
                pass
        
        # 整数
        if text.isdigit():
            return FractionOperand(
                raw=text,
                fraction=Fraction(int(text)),
                fraction_type=FractionType.INTEGER
            )
        
        raise ValueError(f"无法解析分数文本: {text}")
    
    @staticmethod
    def fraction_to_latex(fraction: Fraction, mixed: bool = False) -> str:
        """转换分数为LaTeX格式"""
        if fraction.denominator == 1:
            return str(fraction.numerator)
        
        if mixed and abs(fraction.numerator) >= fraction.denominator:
            whole = fraction.numerator // fraction.denominator
            remainder = abs(fraction.numerator) % fraction.denominator
            if remainder == 0:
                return str(whole)
            return f"#{whole}frac{{{remainder}}}{{{fraction.denominator}}}#"
        
        return f"#frac{{{fraction.numerator}}}{{{fraction.denominator}}}#"
    
    @staticmethod
    def gcd(a: int, b: int) -> int:
        """最大公约数"""
        while b:
            a, b = b, a % b
        return abs(a)
    
    @staticmethod
    def lcm(a: int, b: int) -> int:
        """最小公倍数"""
        return abs(a * b) // FractionUtils.gcd(a, b)


def to_latex(fraction: Fraction) -> str:
    """将 Fraction 转为 LaTeX 格式的字符串"""
    return FractionUtils.fraction_to_latex(fraction)


"""
颠倒数相减实现
两位数或多位数中数字顺序颠倒的数相减
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class ReversedNumberSubtraction(MathCalculator):
    """颠倒数相减"""
    
    def __init__(self):
        super().__init__("颠倒数相减", "两位数颠倒数相减，差是9的倍数", priority=5)
        self.formula = "AB - BA = 9(A-B)"
    
    def _is_reversed_pair(self, a: int, b: int) -> bool:
        """判断是否为颠倒数对"""
        str_a = str(abs(a))
        str_b = str(abs(b))
        
        # 长度必须相同
        if len(str_a) != len(str_b):
            return False
        
        # 检查是否为颠倒关系
        return str_a == str_b[::-1]
    
    def _get_digit_difference(self, num) -> int:
        """获取首位数字与尾位数字的差"""
        str_num = str(abs(int(num)))
        if len(str_num) < 2:
            return 0
        
        first_digit = int(str_num[0])
        last_digit = int(str_num[-1])
        return abs(first_digit - last_digit)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两个数互为颠倒数的减法"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, int) and isinstance(b, int)):
                return False
            
            # 必须都是两位数及以上，且互为颠倒数
            return (abs(a) >= 10 and abs(b) >= 10 and 
                   self._is_reversed_pair(a, b))
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建颠倒数相减步骤"""
        numbers = formula.get_numbers()
        original_a, original_b = [elem.get_numeric_value() for elem in numbers]
        
        # 计算真实结果（保持原始顺序）
        actual_result = original_a - original_b
        
        # 为了分析数字特征，使用绝对值
        a, b = abs(original_a), abs(original_b)
        
        str_a = str(a)
        str_b = str(b)
        
        steps = []
        
        if len(str_a) == 2:
            # 两位数情况
            first_digit = int(str_a[0])
            last_digit = int(str_a[1])
            digit_diff = first_digit - last_digit
            
            # 计算基础差值（不带符号）
            base_result = abs(digit_diff) * 9
            
            steps.append(CalculationStep(
                description=f"识别颠倒数对：{original_a} 和 {original_b}（{str_a} 和 {str_b}）",
                operation="识别模式",
                result="颠倒数相减"
            ))
            
            steps.append(CalculationStep(
                description=f"十位数字 {first_digit}，个位数字 {last_digit}，差为 {digit_diff}",
                operation="计算数字差",
                result=digit_diff
            ))
            
            # 根据原始计算的符号确定结果
            if actual_result >= 0:
                steps.append(CalculationStep(
                    description=f"颠倒数相减 = （首位数字 - 尾位数字）× 9 = {digit_diff} × 9 = {actual_result}",
                    operation="应用公式",
                    result=actual_result,
                    formula="(首位 - 尾位) × 9"
                ))
            else:
                # 负数情况
                steps.append(CalculationStep(
                    description=f"由于 {original_a} < {original_b}，结果为负数",
                    operation="判断符号",
                    result="结果为负"
                ))
                
                steps.append(CalculationStep(
                    description=f"颠倒数相减 = -（首位数字 - 尾位数字）× 9 = -{abs(digit_diff)} × 9 = {actual_result}",
                    operation="应用公式",
                    result=actual_result,
                    formula="负数情况：-(首位 - 尾位) × 9"
                ))
        
        else:
            # 多位数情况（通用公式）
            digit_diff = self._get_digit_difference(a)
            
            steps.append(CalculationStep(
                description=f"识别颠倒数对：{original_a} 和 {original_b}",
                operation="识别模式", 
                result="颠倒数相减"
            ))
            
            steps.append(CalculationStep(
                description=f"首位数字与尾位数字的差：{digit_diff}",
                operation="计算数字差",
                result=digit_diff
            ))
            
            steps.append(CalculationStep(
                description=f"直接计算：{original_a} - {original_b} = {actual_result}",
                operation="直接计算",
                result=actual_result
            ))
            
            if actual_result != 0:
                steps.append(CalculationStep(
                    description=f"验证：{abs(actual_result)} ÷ 9 = {abs(actual_result) // 9}，确实是9的倍数",
                    operation="验证结果",
                    result=f"{abs(actual_result)}是9的{abs(actual_result) // 9}倍"
                ))
        
        return steps


"""
通用除法实现
作为兜底算法处理所有其他除法情况
"""

from typing import List, Union
from decimal import Decimal, getcontext
from fractions import Fraction
from ..base_types import MathCalculator, Formula, CalculationStep

# 设置小数精度
getcontext().prec = 28


class GeneralDivision(MathCalculator):
    """通用除法算法"""
    
    def __init__(self):
        super().__init__("通用除法", "化分简，心算快", priority=1)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：所有除法运算（最低优先级，兜底算法）"""
        if formula.type != "division":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            dividend, divisor = [elem.get_numeric_value() for elem in numbers]
            if divisor == 0:
                return False
            # 支持整数、浮点数
            return isinstance(dividend, (int, float)) and isinstance(divisor, (int, float))
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建通用除法步骤"""
        numbers = formula.get_numbers()
        dividend = numbers[0].get_numeric_value()
        divisor = numbers[1].get_numeric_value()
        
        # 转换Fraction为float以便处理
        if isinstance(dividend, Fraction):
            dividend = float(dividend)
        if isinstance(divisor, Fraction):
            divisor = float(divisor)
        
        # 检查特殊情况
        if divisor == 0:
            return self._construct_zero_division_steps(dividend, divisor)
        elif dividend == 0:
            return self._construct_zero_dividend_steps(dividend, divisor)
        elif divisor == 1:
            return self._construct_one_division_steps(dividend, divisor)
        elif isinstance(dividend, float) or isinstance(divisor, float):
            return self._construct_decimal_steps(dividend, divisor)
        elif dividend < 0 or divisor < 0:
            return self._construct_negative_steps(dividend, divisor)
        else:
            return self._construct_general_steps(int(dividend), int(divisor))
    
    def _construct_zero_division_steps(self, dividend: Union[int, float], divisor: Union[int, float]) -> List[CalculationStep]:
        """构建被零除的错误步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} 检测到除零错误",
            operation="检测除零",
            result="任何数都不能被零除"
        ))
        
        steps.append(CalculationStep(
            description="除零操作是未定义的数学运算",
            operation="数学规则",
            result="错误：除数不能为零",
            formula="a ÷ 0 = 未定义"
        ))
        
        return steps
    
    def _construct_zero_dividend_steps(self, dividend: Union[int, float], divisor: Union[int, float]) -> List[CalculationStep]:
        """构建零被除的步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} 识别零被除规律",
            operation="识别零被除",
            result="零被任何非零数除都等于零"
        ))
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} = 0",
            operation="应用零被除规律",
            result=0,
            formula="0 ÷ a = 0 (a ≠ 0)"
        ))
        
        return steps
    
    def _construct_one_division_steps(self, dividend: Union[int, float], divisor: Union[int, float]) -> List[CalculationStep]:
        """构建被1除的步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} 识别被1除规律",
            operation="识别被1除",
            result="任何数被1除都等于原数"
        ))
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} = {dividend}",
            operation="应用被1除规律",
            result=dividend,
            formula="a ÷ 1 = a"
        ))
        
        return steps
    
    def _construct_negative_steps(self, dividend: Union[int, float], divisor: Union[int, float]) -> List[CalculationStep]:
        """构建负数除法步骤"""
        steps = []
        
        # 判断符号
        is_dividend_negative = dividend < 0
        is_divisor_negative = divisor < 0
        result_positive = not (is_dividend_negative ^ is_divisor_negative)  # 异或取反
        
        abs_dividend = abs(dividend)
        abs_divisor = abs(divisor)
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} 处理负数除法",
            operation="分析符号",
            result="负数除法：同号为正，异号为负"
        ))
        
        steps.append(CalculationStep(
            description=f"符号分析：{dividend} {'< 0' if is_dividend_negative else '> 0'}, {divisor} {'< 0' if is_divisor_negative else '> 0'}",
            operation="确定结果符号",
            result=f"结果为{'正数' if result_positive else '负数'}"
        ))
        
        steps.append(CalculationStep(
            description=f"取绝对值：|{dividend}| = {abs_dividend}, |{divisor}| = {abs_divisor}",
            operation="计算绝对值",
            result=f"计算 {abs_dividend} ÷ {abs_divisor}"
        ))
        
        # 递归处理绝对值除法
        abs_result = abs_dividend / abs_divisor
        
        steps.append(CalculationStep(
            description=f"{abs_dividend} ÷ {abs_divisor} = {abs_result}",
            operation="计算绝对值商",
            result=abs_result
        ))
        
        final_result = abs_result if result_positive else -abs_result
        steps.append(CalculationStep(
            description=f"添加符号：{abs_result} → {final_result}",
            operation="确定最终结果",
            result=final_result,
            formula="负数除法：(-a) ÷ (-b) = ab, (-a) ÷ b = -a/b"
        ))
        
        return steps
    
    def _construct_decimal_steps(self, dividend: Union[int, float], divisor: Union[int, float]) -> List[CalculationStep]:
        """构建小数除法步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} 处理小数除法",
            operation="识别小数",
            result="小数除法：转换为分数或整数计算"
        ))
        
        # 计算结果
        result = dividend / divisor
        
        # 检查是否能转换为简单分数
        try:
            # 尝试将结果转换为分数
            fraction_result = Fraction(result).limit_denominator(100)
            if abs(float(fraction_result) - result) < 0.001 and fraction_result.denominator <= 20:
                steps.append(CalculationStep(
                    description=f"{dividend} ÷ {divisor} = {fraction_result}",
                    operation="分数形式",
                    result=f"分数形式：{fraction_result}"
                ))
                
                # 如果是假分数且能化为带分数
                if fraction_result > 1:
                    whole = fraction_result.numerator // fraction_result.denominator
                    remainder = fraction_result.numerator % fraction_result.denominator
                    if remainder > 0:
                        steps.append(CalculationStep(
                            description=f"化为带分数：{whole} 又 {remainder}/{fraction_result.denominator}",
                            operation="带分数形式",
                            result=f"带分数：{whole} {remainder}/{fraction_result.denominator}"
                        ))
        except:
            pass
        
        steps.append(CalculationStep(
            description=f"小数结果：{dividend} ÷ {divisor} = {result}",
            operation="小数计算",
            result=result,
            formula="小数除法：直接计算或化为分数"
        ))
        
        return steps
    
    def _construct_general_steps(self, dividend: int, divisor: int) -> List[CalculationStep]:
        """构建一般整数除法步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} 使用长除法",
            operation="识别一般除法",
            result="按照长除法步骤计算"
        ))
        
        result = dividend / divisor
        quotient = dividend // divisor
        remainder = dividend % divisor
        
        if remainder == 0:
            # 整除情况
            steps.append(CalculationStep(
                description=f"{dividend} ÷ {divisor} = {quotient}",
                operation="整除计算",
                result=quotient,
                formula="整除：被除数 = 除数 × 商"
            ))
            
            steps.append(CalculationStep(
                description=f"验证：{divisor} × {quotient} = {dividend}",
                operation="验证结果",
                result="✓ 计算正确"
            ))
        else:
            # 有余数情况
            steps.append(CalculationStep(
                description=f"{dividend} ÷ {divisor} = {quotient} 余 {remainder}",
                operation="带余除法",
                result=f"商 {quotient}，余数 {remainder}"
            ))
            
            # 转换为小数形式
            steps.append(CalculationStep(
                description=f"小数形式：{dividend} ÷ {divisor} = {result}",
                operation="小数表示",
                result=result,
                formula="带余除法：被除数 = 除数 × 商 + 余数"
            ))
            
            # 如果可以，提供分数形式
            try:
                fraction = Fraction(dividend, divisor)
                if fraction.denominator <= 20:
                    steps.append(CalculationStep(
                        description=f"分数形式：{dividend} ÷ {divisor} = {fraction}",
                        operation="分数表示",
                        result=f"分数：{fraction}"
                    ))
            except:
                pass
        
        return steps


"""
比值转数值算子 - 形如 a:b=@，求a÷b
"""
from typing import Dict, Any
from fractions import Fraction
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType

class RatioToValueCalculator(FractionCalculator):
    """比值转数值算子"""
    def __init__(self):
        super().__init__("比值转数值", "比值转为数值", priority=2)

    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配比值转数值问题 a:b=@"""
        if problem.operation != OperationType.CONVERT:
            return {"matched": False, "score": 0.0, "reason": "不是转换"}
        if len(problem.operands) < 2:
            return {"matched": False, "score": 0.0, "reason": "需要2个已知数和1个未知数"}
        if problem.id != "ratio_to_value":
            return {"matched": False, "score": 0.0, "reason": "目标格式不是数值"}
        return {"matched": True, "score": 1.0, "reason": "比值转数值"}

    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行比值转数值 a:b=@，即a÷b"""
        try:
            ops = problem.operands
            idx_unknown = [i for i, op in enumerate(ops) if getattr(op, 'raw', None) == '@'][0]
            knowns = [op.fraction.numerator for i, op in enumerate(ops) if i != idx_unknown]
            steps = []
            a, b = knowns
            value = round(a / b, 6)
            steps.append(FractionStep(
                description=f"{a}:{b}，计算{a}÷{b}={value}",
                operation="比值求值",
                result=str(value)
            ))
            return FractionResult(
                success=True,
                result=str(value),
                steps=steps,
                step_count=len(steps),
                formula=f"{a}:{b}=@，{a}÷{b}={value}",
                validation=True
            )
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"比值转数值失败: {str(e)}"
            )



"""
连续数减法实现
处理两个连续整数的减法，结果恒为1
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class ConsecutiveNumberSubtraction(MathCalculator):
    """连续数减法算法"""
    
    def __init__(self):
        super().__init__("连续数减法", "两个连续整数的减法，结果恒为1", priority=7)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两个连续的整数相减"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            minuend, subtrahend = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(minuend, int) and isinstance(subtrahend, int)):
                return False
            
            # 检查是否为连续数（差值为1）
            return (minuend - subtrahend == 1)
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建连续数减法步骤"""
        numbers = formula.get_numbers()
        minuend = int(numbers[0].get_numeric_value())
        subtrahend = int(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 识别连续数减法",
            operation="识别连续数",
            result=f"{minuend}和{subtrahend}是连续整数"
        ))
        
        steps.append(CalculationStep(
            description=f"连续整数相减的差值恒为1",
            operation="应用连续数减法规律",
            result="前数减后数 = 1"
        ))
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} = 1",
            operation="得出结果",
            result=1,
            formula="(n+1) - n = 1"
        ))
        
        return steps


"""
单位换算基础类型定义
"""

from abc import ABC, abstractmethod
from collections.abc import Iterable
from decimal import Decimal
from typing import List, Dict, Any, Optional
from enum import Enum
from dataclasses import dataclass


class UnitType(Enum):
    """单位类型枚举"""
    LENGTH = "length"           # 长度
    AREA = "area"              # 面积
    VOLUME = "volume"          # 体积
    TIME = "time"              # 时间
    MASS = "mass"              # 质量
    CURRENCY = "currency"      # 货币
    NUMBER = "number"          # 数字单位
    PERCENTAGE = "percentage"  # 百分比
    DISCOUNT = "discount"      # 折扣
    UNKNOWN = "unknown"        # 未知类型


@dataclass
class UnitValue:
    """单位值"""
    value: float
    unit: str
    operator: str = "+"
    
    def __str__(self):
        # 先判断value是否数值，如果self.value可以是整数，则显示为整数
        if not isinstance(self.value, (int, float)):
            return f"{self.value}{self.unit}"
        if abs(self.value - round(self.value)) < 1e-10:
            return f"{int(round(self.value))}{self.unit}"
        return f"{self.value}{self.unit}"

@dataclass
class ConversionStep:
    """单位换算步骤"""
    description: str
    operation: str
    from_value: UnitValue
    result: UnitValue  # 修改字段名以保持一致性
    formula: Optional[str] = None
    
    def __str__(self):
        return f"{self.description}: {self.from_value} → {self.result}"
    def to_dict(self) -> Dict[str, Any]:
        result = self.result
        if isinstance(result, UnitValue):
            result = f"{result.value}{result.unit}"
        return {
            "description": self.description,
            "operation": self.operation,
            "from_value": str(self.from_value),
            "result": result,
            "formula": self.formula
        }


@dataclass
class UnitProblem:
    """单位换算题目"""
    original_text: str
    unit_type: UnitType
    source_values: List[UnitValue]
    target_unit: str
    
    def __str__(self):
        return self.original_text


class UnitCalculator(ABC):
    """单位换算算子基类"""
    
    def __init__(self, name: str, description: str, unit_type: UnitType):
        self.name = name
        self.description = description
        self.unit_type = unit_type
        self.conversion_rules = self._initialize_conversion_rules()
    
    def _round_result(self, value: float, precision: int = 10) -> float:
        """使用Decimal进行高精度四舍五入"""
        try:
            # 使用Decimal进行高精度计算
            decimal_value = Decimal(str(value))
            # 四舍五入到指定精度
            rounded = round(float(decimal_value), precision)
            
            # 如果结果接近整数，返回整数
            if abs(rounded - round(rounded)) < 1e-10:
                return int(round(rounded))
            return rounded
        except:
            # 如果Decimal转换失败，使用普通四舍五入
            return round(value, precision)
    
    @abstractmethod
    def _initialize_conversion_rules(self) -> Dict[str, Dict[str, float]]:
        """初始化单位换算规则"""
        pass
    
    @abstractmethod
    def is_match_pattern(self, problem: UnitProblem) -> bool:
        """判断是否匹配当前算子"""
        pass
    
    @abstractmethod
    def solve(self, problem: UnitProblem) -> tuple[float, List[ConversionStep]]:
        """求解单位换算问题，返回(结果, 步骤列表)"""
        pass
    
    def get_conversion_factor(self, from_unit: str, to_unit: str) -> Optional[float]:
        """获取两个单位之间的换算因子"""
        if from_unit in self.conversion_rules:
            return self.conversion_rules[from_unit].get(to_unit)
        return None
    
    def validate_result(self, problem: UnitProblem, result: float) -> bool:
        """验证计算结果"""
        try:
            # 简单验证：重新计算一遍
            calculated_result, _ = self.solve(problem)
            return abs(calculated_result - result) < 1e-10
        except:
            return False



"""
拆分除法实现
将被除数拆分为方便计算的和，再分别除以除数
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class SplitDivision(MathCalculator):
    """拆分除法算法"""
    
    def __init__(self):
        super().__init__("拆分除法", "拆分除，再相加", priority=4)
    
    def _find_split_candidates(self, dividend: int, divisor: int) -> List[tuple]:
        """找到合适的拆分方案"""
        candidates = []

        # 策略1：按整十数拆分
        if dividend >= 10:
            tens_part = (dividend // 10) * 10
            ones_part = dividend % 10
            
            if tens_part > 0 and ones_part >= 0:
                # 检查是否能整除
                tens_quotient = tens_part / divisor
                ones_quotient = ones_part / divisor
                
                # 优先选择能整除的拆分
                if tens_part % divisor == 0 or ones_part % divisor == 0:
                    candidates.append((tens_part, ones_part, "整十数拆分", 1))
                else:
                    candidates.append((tens_part, ones_part, "整十数拆分", 3))
        
        # 策略2：按倍数拆分（寻找除数的倍数）
        for multiplier in range(1, dividend // divisor + 1):
            multiple = divisor * multiplier
            if multiple < dividend:
                remainder = dividend - multiple
                if remainder > 0:
                    # 完美倍数拆分，优先级最高
                    candidates.append((multiple, remainder, f"{divisor}的{multiplier}倍拆分", 0))
        
        # 策略3：按相近数值拆分（使两部分接近）
        if dividend > divisor * 2:
            half = dividend // 2
            # 找到最接近一半且便于计算的拆分点
            for offset in range(min(10, half)):
                part1 = half + offset
                part2 = dividend - part1
                if part1 > 0 and part2 > 0:
                    candidates.append((part1, part2, "均匀拆分", 5))
                
                if offset > 0:
                    part1 = half - offset
                    part2 = dividend - part1
                    if part1 > 0 and part2 > 0:
                        candidates.append((part1, part2, "均匀拆分", 5))
        
        # 按优先级排序
        candidates.sort(key=lambda x: (x[3], abs(x[0] - x[1])))
        return candidates[:3]  # 返回最好的3个候选
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：被除数可以有效拆分的除法"""
        if formula.type != "division":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        dividend, divisor = [elem.get_numeric_value() for elem in numbers]
        ## 如果被除数是5的倍数，可使用更高阶方法
        if dividend % 5 == 0:
            return False
        
        ## 如果结果是10的倍数，可使用更高阶方法
        if divisor != 0 and dividend % (divisor * 10) == 0:
            return False
        try:
            dividend, divisor = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(dividend, int) and isinstance(divisor, int)):
                return False
            
            if divisor == 0 or dividend <= 0:
                return False
            
            # 被除数应该足够大，适合拆分（至少是除数的2倍以上）
            if dividend < divisor * 2:
                return False
            
            # 被除数应该小于1000（避免过于复杂）
            if dividend > 999:
                return False
            
            # 除数应该是单位数或两位数（便于心算）
            if divisor > 99:
                return False
            
            # 检查是否有合适的拆分方案
            candidates = self._find_split_candidates(dividend, divisor)
            return len(candidates) > 0
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建拆分除法步骤"""
        numbers = formula.get_numbers()
        dividend, divisor = [elem.get_numeric_value() for elem in numbers]
        
        # 确保是整数
        dividend_int = int(dividend)
        divisor_int = int(divisor)
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"识别拆分除法：{dividend_int} ÷ {divisor_int}",
            operation="识别模式",
            result="拆分除，再相加"
        ))
        
        # 找到最佳拆分方案
        candidates = self._find_split_candidates(dividend_int, divisor_int)
        if not candidates:
            # 如果没有找到拆分方案，使用默认拆分
            part1 = (dividend_int // 10) * 10
            part2 = dividend_int % 10
            split_method = "默认拆分"
        else:
            part1, part2, split_method, _ = candidates[0]
        
        steps.append(CalculationStep(
            description=f"选择拆分方案（{split_method}）：{dividend_int} = {part1} + {part2}",
            operation="确定拆分",
            result=f"{dividend_int} = {part1} + {part2}"
        ))
        
        # 分别计算两部分的除法
        quotient1 = part1 / divisor_int
        quotient2 = part2 / divisor_int
        
        steps.append(CalculationStep(
            description=f"分别计算：{part1} ÷ {divisor_int} = {quotient1}",
            operation="第一部分",
            result=quotient1
        ))
        
        steps.append(CalculationStep(
            description=f"分别计算：{part2} ÷ {divisor_int} = {quotient2}",
            operation="第二部分",
            result=quotient2
        ))
        
        # 合并结果
        final_result = quotient1 + quotient2
        
        steps.append(CalculationStep(
            description=f"合并结果：{quotient1} + {quotient2} = {final_result}",
            operation="合并结果",
            result=final_result,
            formula="拆分除法：(A+B) ÷ C = A ÷ C + B ÷ C"
        ))
        
        # 如果结果是分数，提供额外说明
        if isinstance(final_result, float) and not final_result.is_integer():
            steps.append(CalculationStep(
                description=f"验证：{dividend_int} ÷ {divisor_int} = {final_result}",
                operation="验证结果",
                result=f"结果为小数：{final_result}"
            ))
        
        return steps


"""
接近整百数的乘法实现
适用于接近任意整百数（100、200、300等）的数相乘，使用补数简化计算
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class NearHundredsMultiplication(MathCalculator):
    """接近整百数的乘法"""
    
    def __init__(self):
        super().__init__("接近整百数乘法", "接近整百数相乘，使用补数简化计算", priority=3)
        self.formula = "(N - a)(N - b) = N² - N(a + b) + ab"
    
    def _find_nearest_hundred(self, num) -> tuple:
        """找到最接近的整百数，返回(整百数, 补数)"""
        num = int(num)
        # 找到最接近的整百数
        nearest_hundred = round(num / 100) * 100
        if nearest_hundred == 0:
            nearest_hundred = 100
        
        complement = nearest_hundred - num
        
        # 只有当补数绝对值小于等于25时才认为接近
        if abs(complement) <= 25:
            return nearest_hundred, complement
        return None, 0
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两个接近整百数的数相乘"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, int) and isinstance(b, int)):
                return False
            
            # 尝试找到一个合适的整百数基准
            possible_bases = [100, 200, 300, 400, 500, 600]  # 可以根据需要扩展
            
            for base in possible_bases:
                comp_a = abs(base - a)
                comp_b = abs(base - b)
                # 两个补数都必须小于等于25，且至少有一个小于等于15
                if comp_a <= 25 and comp_b <= 25 and (comp_a <= 15 or comp_b <= 15):
                    return True
            
            return False
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建接近整百数乘法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 找到最接近的整百数
        nearest_a, complement_a = self._find_nearest_hundred(a)
        nearest_b, complement_b = self._find_nearest_hundred(b)
        
        # 选择更适合的基准整百数（选择让两个补数绝对值之和最小的）
        if nearest_a == nearest_b:
            base_hundred = nearest_a
        else:
            # 计算使用不同基准时的补数绝对值之和
            base_hundred_a = nearest_a
            base_hundred_b = nearest_b
            
            # 使用nearest_a作为基准时的补数绝对值之和
            comp_a_with_base_a = abs(base_hundred_a - a)
            comp_b_with_base_a = abs(base_hundred_a - b)
            total_diff_a = comp_a_with_base_a + comp_b_with_base_a
            
            # 使用nearest_b作为基准时的补数绝对值之和
            comp_a_with_base_b = abs(base_hundred_b - a)
            comp_b_with_base_b = abs(base_hundred_b - b)
            total_diff_b = comp_a_with_base_b + comp_b_with_base_b
            
            # 选择补数绝对值之和最小的基准
            if total_diff_a <= total_diff_b:
                base_hundred = base_hundred_a
            else:
                base_hundred = base_hundred_b
            
            # 如果选择的基准导致某个补数超过25，则不适用此算法
            comp_a_final = base_hundred - a
            comp_b_final = base_hundred - b
            if abs(comp_a_final) > 25 or abs(comp_b_final) > 25:
                # 尝试其他可能的基准
                possible_bases = [100, 200, 300, 400, 500]  # 可以扩展
                best_base = None
                min_total_diff = float('inf')
                
                for base in possible_bases:
                    comp_a_test = abs(base - a)
                    comp_b_test = abs(base - b)
                    if comp_a_test <= 25 and comp_b_test <= 25:
                        total_diff = comp_a_test + comp_b_test
                        if total_diff < min_total_diff:
                            min_total_diff = total_diff
                            best_base = base
                
                if best_base:
                    base_hundred = best_base
        
        # 重新计算基于选定基准的补数
        complement_a = base_hundred - a
        complement_b = base_hundred - b
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"识别接近整百数：{a} 和 {b} 都接近 {base_hundred}",
            operation="识别模式",
            result=f"使用接近{base_hundred}的乘法公式"
        ))
        
        # 显示补数
        steps.append(CalculationStep(
            description=f"{a} 距离 {base_hundred} 的补数：{complement_a}，{b} 距离 {base_hundred} 的补数：{complement_b}",
            operation="计算补数",
            result=f"a={complement_a}, b={complement_b}"
        ))
        
        # 应用公式：(base-comp_a)(base-comp_b) = base² - base(comp_a + comp_b) + comp_a×comp_b
        sum_complements = complement_a + complement_b
        product_complements = complement_a * complement_b
        base_squared = base_hundred * base_hundred
        
        # 处理补数和的显示
        if complement_a < 0 and complement_b < 0:
            sum_description = f"补数和：({complement_a}) + ({complement_b}) = {sum_complements}"
        elif complement_a < 0:
            sum_description = f"补数和：({complement_a}) + {complement_b} = {sum_complements}"
        elif complement_b < 0:
            sum_description = f"补数和：{complement_a} + ({complement_b}) = {sum_complements}"
        else:
            sum_description = f"补数和：{complement_a} + {complement_b} = {sum_complements}"
        
        steps.append(CalculationStep(
            description=sum_description,
            operation="计算补数和",
            result=sum_complements
        ))
        
        # 处理补数积的显示
        if complement_a < 0 and complement_b < 0:
            product_description = f"补数积：({complement_a}) × ({complement_b}) = {product_complements}"
        elif complement_a < 0:
            product_description = f"补数积：({complement_a}) × {complement_b} = {product_complements}"
        elif complement_b < 0:
            product_description = f"补数积：{complement_a} × ({complement_b}) = {product_complements}"
        else:
            product_description = f"补数积：{complement_a} × {complement_b} = {product_complements}"
        
        steps.append(CalculationStep(
            description=product_description,
            operation="计算补数积",
            result=product_complements
        ))
        
        # 计算最终结果
        hundred_times_sum = base_hundred * sum_complements
        intermediate = base_squared - hundred_times_sum
        final_result = intermediate + product_complements
        
        # 构建公式显示，处理负数情况
        if sum_complements < 0:
            sum_part = f"({sum_complements})"
        else:
            sum_part = str(sum_complements)
        
        if product_complements < 0:
            product_part = f"({product_complements})"
        else:
            product_part = str(product_complements)
        
        steps.append(CalculationStep(
            description=f"应用公式：{base_squared} - {base_hundred}×{sum_part} + {product_part}",
            operation="应用公式",
            result=f"{base_squared} - {hundred_times_sum} + {product_complements}" if hundred_times_sum >= 0 else f"{base_squared} - ({hundred_times_sum}) + {product_complements}"
        ))
        
        # 处理最终计算步骤的显示
        if hundred_times_sum < 0 and product_complements < 0:
            calculation_display = f"= {base_squared} - ({hundred_times_sum}) + ({product_complements}) = {base_squared} + {-hundred_times_sum} - {-product_complements} = {final_result}"
        elif hundred_times_sum < 0:
            calculation_display = f"= {base_squared} - ({hundred_times_sum}) + {product_complements} = {base_squared} + {-hundred_times_sum} + {product_complements} = {final_result}"
        elif product_complements < 0:
            calculation_display = f"= {base_squared} - {hundred_times_sum} + ({product_complements}) = {base_squared} - {hundred_times_sum} - {-product_complements} = {final_result}"
        else:
            calculation_display = f"= {intermediate} + {product_complements} = {final_result}"
        
        steps.append(CalculationStep(
            description=calculation_display,
            operation="计算结果",
            result=final_result,
            formula=f"({base_hundred}-a)({base_hundred}-b) = {base_hundred}² - {base_hundred}(a+b) + ab"
        ))
        
        return steps


"""
运算后比较算子 - 先计算表达式结果再比较大小
"""

from typing import Dict, Any
from fractions import Fraction
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType
from ..parser import FractionParser


class ExpressionCompareCalculator(FractionCalculator):
    """运算后比较算子"""
    
    def __init__(self):
        super().__init__("运算后比较", "先计算表达式结果再比较大小", priority=5)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配运算后比较问题"""
        # 这个算子通常由引擎特别调用，处理复合表达式比较
        # 例如：1/3 + 1/2 @ 2/3
        if problem.operation == OperationType.COMPARE:
            # 检查是否有复合表达式（原始文本包含运算符）
            original = problem.original_text
            if any(op in original.split('@')[0] for op in ['+', '-', '×', '÷']):
                return {"matched": True, "score": 0.7, "reason": "运算后比较"}
        
        return {"matched": False, "score": 0.0, "reason": "不是运算后比较"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行运算后比较"""
        try:
            steps = []
            
            # 解析原始表达式的两侧
            original = problem.original_text
            if '@' not in original:
                raise ValueError("表达式格式错误")
            
            left_expr, right_expr = original.split('@')
            left_expr = left_expr.strip()
            right_expr = right_expr.strip()
            
            steps.append(FractionStep(
                description=f"需要比较：{left_expr} 与 {right_expr}",
                operation="识别比较表达式",
                result="先分别计算两侧的值"
            ))
            
            # 计算左侧表达式
            left_result = self._evaluate_expression(left_expr, steps)
            
            # 计算右侧表达式
            right_result = self._evaluate_expression(right_expr, steps)
            
            # 比较结果
            steps.append(FractionStep(
                description=f"比较计算结果：{left_result} 与 {right_result}",
                operation="比较结果",
                result="使用分数比较方法"
            ))
            
            # 执行比较
            if left_result > right_result:
                comparison = ">"
                explanation = f"因为 {left_result} > {right_result}"
            elif left_result < right_result:
                comparison = "<"
                explanation = f"因为 {left_result} < {right_result}"
            else:
                comparison = "="
                explanation = f"因为 {left_result} = {right_result}"
            
            steps.append(FractionStep(
                description=explanation,
                operation="得出结论",
                result=f"{left_expr} {comparison} {right_expr}",
                formula=f"{left_result} {comparison} {right_result}"
            ))
            
            return FractionResult(
                success=True,
                result=comparison,
                steps=steps,
                step_count=len(steps),
                formula=f"运算后比较：{left_expr} {comparison} {right_expr}",
                validation=True
            )
            
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"运算后比较失败: {str(e)}"
            )
    
    def _evaluate_expression(self, expr: str, steps: list) -> Fraction:
        """计算表达式的值"""
        try:
            # 简单解析和计算表达式
            if '+' in expr:
                parts = expr.split('+')
                fractions = [self._parse_fraction(part.strip()) for part in parts]
                result = sum(fractions, Fraction(0))
                
                steps.append(FractionStep(
                    description=f"计算 {expr} = {' + '.join(str(f) for f in fractions)} = {result}",
                    operation="计算加法",
                    result=str(result)
                ))
                
            elif '-' in expr:
                parts = expr.split('-')
                fractions = [self._parse_fraction(part.strip()) for part in parts if part.strip()]
                result = fractions[0] - sum(fractions[1:], Fraction(0))
                
                steps.append(FractionStep(
                    description=f"计算 {expr} = {result}",
                    operation="计算减法",
                    result=str(result)
                ))
                
            elif '×' in expr or '*' in expr:
                op = '×' if '×' in expr else '*'
                parts = expr.split(op)
                fractions = [self._parse_fraction(part.strip()) for part in parts]
                result = fractions[0]
                for f in fractions[1:]:
                    result *= f
                
                steps.append(FractionStep(
                    description=f"计算 {expr} = {result}",
                    operation="计算乘法",
                    result=str(result)
                ))
                
            elif '÷' in expr or '/' in expr:
                op = '÷' if '÷' in expr else '/'
                parts = expr.split(op)
                fractions = [self._parse_fraction(part.strip()) for part in parts]
                result = fractions[0]
                for f in fractions[1:]:
                    result /= f
                
                steps.append(FractionStep(
                    description=f"计算 {expr} = {result}",
                    operation="计算除法",
                    result=str(result)
                ))
                
            else:
                # 单个分数
                result = self._parse_fraction(expr)
                
                steps.append(FractionStep(
                    description=f"解析分数 {expr} = {result}",
                    operation="解析分数",
                    result=str(result)
                ))
            
            return result
            
        except Exception as e:
            raise ValueError(f"无法计算表达式 {expr}: {str(e)}")
    
    def _parse_fraction(self, text: str) -> Fraction:
        """解析分数文本为Fraction对象"""
        from ..base_types import FractionUtils
        operand = FractionUtils.parse_fraction_text(text)
        return operand.fraction



"""
破十法实现
20以内退位减法，将被减数拆分成10和剩余部分
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class BreakTenSubtraction(MathCalculator):
    """破十法（20以内退位减法）"""
    
    def __init__(self):
        super().__init__("破十法", "20以内退位减法，将被减数拆分成10和剩余部分", priority=2)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：20以内的退位减法"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            # 20以内的整数，且需要退位（被减数个位小于减数）
            return (isinstance(a, int) and isinstance(b, int) and 
                   10 <= a <= 20 and 0 < b <= 10 and a % 10 < b)
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建破十法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 拆分被减数
        tens = (a // 10) * 10
        ones = a % 10
        
        steps = [
            CalculationStep(
                description=f"将 {a} 拆分为 {tens} 和 {ones}",
                operation=f"拆分被减数",
                result=f"{a} = {tens} + {ones}"
            ),
            CalculationStep(
                description=f"先算 {tens} - {b} = {tens - b}",
                operation=f"用整十数减",
                result=tens - b
            ),
            CalculationStep(
                description=f"再算 {tens - b} + {ones} = {tens - b + ones}",
                operation=f"加上个位数",
                result=tens - b + ones
            )
        ]
        
        return steps


"""
百分比和折扣换算算子
处理数字/小数与百分比、折扣之间的换算
"""

from typing import List, Dict
from decimal import Decimal, getcontext
from ..base_types import UnitCalculator, UnitProblem, UnitType, UnitValue, ConversionStep

# 设置高精度计算
getcontext().prec = 28

cn_chars = '零一二三四五六七八九十'

class PercentageDiscountCalculator(UnitCalculator):
    """百分比和折扣换算算子"""
    
    def __init__(self):
        super().__init__("百分比折扣换算", "处理数字与百分比、折扣之间的换算", UnitType.PERCENTAGE)
    
    def _initialize_conversion_rules(self) -> Dict[str, Dict[str, float]]:
        """初始化百分比折扣换算规则"""
        return {
            # 小数到百分比：×100
            "": {"%": 100, "折": 10},
            "个": {"%": 100, "折": 10},
            
            # 百分比到小数：÷100
            "%": {"": 0.01, "个": 0.01, "折": 0.1},
            
            # 折扣到小数：÷10
            "折": {"": 0.1, "个": 0.1, "%": 10},
            "成": {"折": 1, "": 0.1, "个": 0.1, "%": 10}
        }
    
    def is_match_pattern(self, problem: UnitProblem) -> bool:
        """判断是否为百分比折扣换算问题"""
        if problem.unit_type not in [UnitType.PERCENTAGE, UnitType.DISCOUNT]:
            return False
        
        # 检查是否涉及百分比或折扣单位
        source_units = [sv.unit for sv in problem.source_values]
        target_unit = problem.target_unit
        
        percentage_discount_units = ["", "个", "%", "折", "成"]
        return (any(unit in percentage_discount_units for unit in source_units) and 
                target_unit in percentage_discount_units)
    
    def solve(self, problem: UnitProblem) -> tuple[float, List[ConversionStep]]:
        """求解百分比折扣换算问题"""
        steps = []
        total_result = Decimal('0')
        
        for source_value in problem.source_values:
            # 处理空单位，统一为""
            from_unit = source_value.unit if source_value.unit else ""
            to_unit = problem.target_unit if problem.target_unit else ""
            
            # 获取换算系数
            conversion_factor = self._get_conversion_factor(from_unit, to_unit)
            
            if conversion_factor is None:
                raise ValueError(f"无法从{from_unit or '数字'}换算到{to_unit or '数字'}")
            
            # 使用Decimal进行高精度计算
            source_decimal = Decimal(str(source_value.value))
            factor_decimal = Decimal(str(conversion_factor))
            converted_decimal = source_decimal * factor_decimal
            
            converted_value = float(converted_decimal)
            total_result += converted_decimal
            
            # 添加换算原理说明
            principle_step = self._create_principle_explanation(from_unit, to_unit, conversion_factor)
            if principle_step:
                steps.append(principle_step)
            converted_value = self._round_result(converted_value)
            
            # 生成换算步骤
            from_display = f"{source_value.value}{from_unit}" if from_unit else str(source_value.value)
            to_display = f"{converted_value}{to_unit}" if to_unit else str(converted_value)
            
            step = ConversionStep(
                description=f"计算：{from_display} → {to_display}",
                operation="百分比折扣换算",
                from_value=source_value,
                result=UnitValue(converted_value, to_unit),
                formula=f"{source_value.value} × {conversion_factor} = {converted_value}"
            )
            steps.append(step)
        
        # 如果有多个源值，添加求和步骤
        if len(problem.source_values) > 1:
            step = ConversionStep(
                description=f"合并换算结果",
                operation="求和",
                from_value=UnitValue(0, problem.target_unit),
                result=UnitValue(float(total_result), problem.target_unit),
                formula=f"总计 = {' + '.join([str(v.value) for v in problem.source_values])}"
            )
            steps.append(step)
        
        # 四舍五入结果
        final_result = self._round_result(float(total_result))
        
        return final_result, steps
    
    def _get_conversion_factor(self, from_unit: str, to_unit: str) -> float:
        """获取换算系数"""
        conversion_rules = self._initialize_conversion_rules()
        
        # 处理空单位
        from_unit = from_unit if from_unit else ""
        to_unit = to_unit if to_unit else ""
        
        if from_unit in conversion_rules and to_unit in conversion_rules[from_unit]:
            return conversion_rules[from_unit][to_unit]
        
        return None
    
    def _create_principle_explanation(self, from_unit: str, to_unit: str, factor: float) -> ConversionStep:
        """创建换算原理说明步骤"""
        explanations = {
            ("", "%"): "小数转百分比：乘以100 (移动小数点2位)",
            ("个", "%"): "整数转百分比：乘以100",
            ("", "折"): "小数转折扣：乘以10 (移动小数点1位)",
            ("个", "折"): "整数转折扣：乘以10",
            ("%", ""): "百分比转小数：除以100 (移动小数点2位)",
            ("%", "个"): "百分比转整数：除以100",
            ("折", ""): "折扣转小数：除以10 (移动小数点1位)",
            ("折", "个"): "折扣转整数：除以10",
            ("%", "折"): "百分比转折扣：除以10",
            ("折", "%"): "折扣转百分比：乘以10"
        }
        
        key = (from_unit, to_unit)
        if key in explanations:
            return ConversionStep(
                description=explanations[key],
                operation="换算原理说明",
                from_value=UnitValue(1, from_unit),
                result=UnitValue(factor, to_unit),
                formula=f"换算系数：{factor}"
            )
        
        return None
    
    def _round_result(self, value: float) -> float:
        """四舍五入结果，避免浮点数精度问题"""
        # 对于百分比和折扣，保留4位小数精度
        return round(value, 4)


class PercentageCalculator(PercentageDiscountCalculator):
    """百分比换算算子"""
    
    def __init__(self):
        super().__init__()
        self.name = "百分比换算"
        self.description = "处理数字与百分比之间的换算"
        self.unit_type = UnitType.PERCENTAGE
    
    def is_match_pattern(self, problem: UnitProblem) -> bool:
        """匹配百分比换算问题"""
        if problem.unit_type != UnitType.PERCENTAGE:
            return False
        
        source_units = [sv.unit for sv in problem.source_values]
        target_unit = problem.target_unit
        
        # 涉及百分比的换算
        return ("%" in source_units or target_unit == "%") and target_unit != "折"


class DiscountCalculator(PercentageDiscountCalculator):
    """折扣换算算子"""
    
    def __init__(self):
        super().__init__()
        self.name = "折扣换算"
        self.description = "处理数字与折扣之间的换算"
        self.unit_type = UnitType.DISCOUNT
    
    def is_match_pattern(self, problem: UnitProblem) -> bool:
        """匹配折扣换算问题"""
        if problem.unit_type != UnitType.DISCOUNT:
            return False
        
        source_units = [sv.unit for sv in problem.source_values]
        target_unit = problem.target_unit
        
        # 涉及折扣的换算
        return ("折" in source_units or target_unit == "折") and target_unit != "%"



"""
乘以10的幂实现 (补零法)
任何数乘以10, 100, 1000...，只需在数字后面添加相应个数的0
支持所有整零数：20, 30, 400, 500, 1000, 2000等
优化版本：同时分析两个乘数的零，统一处理
"""

from typing import List, Tuple
from ..base_types import MathCalculator, Formula, CalculationStep


class MultiplyByPowersOfTen(MathCalculator):
    """乘以10的幂算法 (补零法)"""
    
    def __init__(self):
        super().__init__("补零法", "乘以整十、整百、整千等数，利用补零规律简化计算", priority=5)
    
    def _analyze_zero_number(self, num) -> Tuple[bool, int, int]:
        """分析是否为整零数，返回(是否匹配, 非零部分, 零的个数)"""
        if not isinstance(num, int) or num <= 0:
            return False, 0, 0
        
        zero_count = 0
        temp = num
        
        # 计算末尾零的个数
        while temp % 10 == 0 and temp > 0:
            temp //= 10
            zero_count += 1
        
        # 如果有末尾零，说明是整零数
        return zero_count > 0, temp, zero_count
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：至少有一个整零数的乘法"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, (int, float)) and isinstance(b, (int, float))):
                return False
            
            # 检查是否有整零数
            is_zero_a, _, _ = self._analyze_zero_number(int(a)) if isinstance(a, int) else (False, 0, 0)
            is_zero_b, _, _ = self._analyze_zero_number(int(b)) if isinstance(b, int) else (False, 0, 0)
            
            return is_zero_a or is_zero_b
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建补零法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 分析两个数的零情况
        is_zero_a, non_zero_a, zero_count_a = self._analyze_zero_number(int(a)) if isinstance(a, int) else (False, int(a), 0)
        is_zero_b, non_zero_b, zero_count_b = self._analyze_zero_number(int(b)) if isinstance(b, int) else (False, int(b), 0)
        
        # 如果都不是整数或都没有末尾零，使用原数
        if not is_zero_a:
            non_zero_a, zero_count_a = a, 0
        if not is_zero_b:
            non_zero_b, zero_count_b = b, 0
        
        total_zero_count = zero_count_a + zero_count_b
        
        steps = []
        
        # 识别补零法
        zero_descriptions = []
        if zero_count_a > 0:
            zero_descriptions.append(f"{a}有{zero_count_a}个零")
        if zero_count_b > 0:
            zero_descriptions.append(f"{b}有{zero_count_b}个零")
        
        if zero_descriptions:
            steps.append(CalculationStep(
                description=f"{a} × {b} 使用补零法",
                operation="识别补零法",
                result=f"发现整零数：{', '.join(zero_descriptions)}"
            ))
        
        # 分解过程
        decomposition_parts = []
        if zero_count_a > 0:
            decomposition_parts.append(f"{a} = {non_zero_a} × 10^{zero_count_a}")
        else:
            decomposition_parts.append(f"{a} = {non_zero_a}")
            
        if zero_count_b > 0:
            decomposition_parts.append(f"{b} = {non_zero_b} × 10^{zero_count_b}")
        else:
            decomposition_parts.append(f"{b} = {non_zero_b}")
        
        steps.append(CalculationStep(
            description=f"分解：{'; '.join(decomposition_parts)}",
            operation="分解整零数",
            result=f"非零部分：{non_zero_a} × {non_zero_b}，总共补{total_zero_count}个零"
        ))
        
        # 计算非零部分乘法
        non_zero_result = non_zero_a * non_zero_b
        if non_zero_a != 1 and non_zero_b != 1:
            steps.append(CalculationStep(
                description=f"计算非零部分：{non_zero_a} × {non_zero_b} = {non_zero_result}",
                operation="非零部分乘法",
                result=non_zero_result
            ))
        elif non_zero_a == 1 or non_zero_b == 1:
            steps.append(CalculationStep(
                description=f"非零部分：{non_zero_a} × {non_zero_b} = {non_zero_result}",
                operation="简单乘法",
                result=non_zero_result
            ))
        
        # 补零操作
        if total_zero_count > 0:
            final_result = non_zero_result * (10 ** total_zero_count)
            if isinstance(non_zero_result, int):
                result_str = str(non_zero_result) + '0' * total_zero_count
            else:
                result_str = str(final_result)
            
            steps.append(CalculationStep(
                description=f"补零：{non_zero_result} 后面补{total_zero_count}个零 = {result_str}",
                operation="补零操作",
                result=result_str
            ))
        else:
            final_result = non_zero_result
        
        steps.append(CalculationStep(
            description=f"最终结果：{final_result}",
            operation="确认结果",
            result=final_result,
            formula="补零法：提取所有零，计算非零部分乘积，再补相应个数的零"
        ))
        
        return steps


"""
求分数倒数等式算子 - 处理 #frac{@}{@}#×#frac{58}{9}#=1 类型题目
"""

from typing import Dict, Any
from fractions import Fraction
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType


class ReciprocalEquationCalculator(FractionCalculator):
    """求分数倒数等式算子"""
    
    def __init__(self):
        super().__init__("求倒数等式", "根据乘积为1求未知分数", priority=9)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配求倒数等式问题"""
        # 匹配 a × b = 1 类型，其中一个是未知数
        if (problem.operation == OperationType.MULTIPLY and 
            len(problem.operands) == 1 and 
            problem.target_format in ("reciprocal", 'fraction')):
            return {"matched": True, "score": 1.0, "reason": "求倒数等式"}
        
        # 匹配 1 ÷ a = ? 类型
        if (problem.operation == OperationType.DIVIDE and 
            len(problem.operands) >= 2 and
            problem.operands[0].fraction == Fraction(1)):
            return {"matched": True, "score": 1.0, "reason": "1除以分数求倒数"}
        
        return {"matched": False, "score": 0.0, "reason": "不是求倒数等式"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行求倒数等式计算"""
        try:
            steps = []
            
            if problem.operation == OperationType.MULTIPLY:
                # 处理 ? × a = 1 类型
                known_fraction = problem.operands[0].fraction
                
                steps.append(FractionStep(
                    description=f"根据倒数定义：两个数相乘等于1，则互为倒数",
                    operation="倒数定义",
                    result="需要找到与已知分数相乘等于1的分数"
                ))
                
                steps.append(FractionStep(
                    description=f"已知分数：{known_fraction}",
                    operation="确定已知条件",
                    result=str(known_fraction)
                ))
                
                # 求倒数：交换分子分母
                reciprocal = Fraction(known_fraction.denominator, known_fraction.numerator)
                
                steps.append(FractionStep(
                    description=f"分数倒数技巧：分子分母互换位置",
                    operation="交换分子分母",
                    result=f"{known_fraction} → {reciprocal}",
                    formula="分数的倒数 = 分母/分子"
                ))
                
                # 验证
                steps.append(FractionStep(
                    description=f"验证：{reciprocal} × {known_fraction} = {reciprocal * known_fraction}",
                    operation="验证倒数",
                    result="乘积为1，验证正确",
                    formula=f"{reciprocal} × {known_fraction} = 1"
                ))
                
                return FractionResult(
                    success=True,
                    result=reciprocal,
                    steps=steps,
                    step_count=len(steps),
                    formula=f"求{known_fraction}的倒数 = {reciprocal}",
                    validation=True
                )
                
            elif problem.operation == OperationType.DIVIDE:
                # 处理 1 ÷ a = ? 类型
                dividend = problem.operands[0].fraction  # 1
                divisor = problem.operands[1].fraction   # 要求倒数的分数
                
                steps.append(FractionStep(
                    description=f"1除以一个数的结果就是这个数的倒数",
                    operation="倒数定义",
                    result="1 ÷ 分数 = 分数的倒数"
                ))
                
                steps.append(FractionStep(
                    description=f"要求：1 ÷ {divisor}",
                    operation="确定问题",
                    result=f"求{divisor}的倒数"
                ))
                
                # 交换分子分母求倒数
                reciprocal = Fraction(divisor.denominator, divisor.numerator)
                
                steps.append(FractionStep(
                    description=f"分数倒数：交换分子分母",
                    operation="求倒数",
                    result=f"{divisor} → {reciprocal}",
                    formula="分数的倒数 = 分母/分子"
                ))
                
                # 验证
                steps.append(FractionStep(
                    description=f"验证：1 ÷ {divisor} = {reciprocal}",
                    operation="验证计算",
                    result=str(reciprocal),
                    formula=f"1 ÷ {divisor} = {reciprocal}"
                ))
                
                return FractionResult(
                    success=True,
                    result=reciprocal,
                    steps=steps,
                    step_count=len(steps),
                    formula=f"1 ÷ {divisor} = {reciprocal}",
                    validation=True
                )
            
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"求倒数等式计算失败: {str(e)}"
            )



"""
Command-line interface for kousuan package
"""
import argparse
from fractions import Fraction
import sys
import json
import os

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../../')))

from kousuan.skills import SmartCalculatorEngine

# 导入单位换算引擎
from kousuan.units.unit_engine import UnitConversionEngine

from kousuan.fraction.engine import FractionEngine

# 导入混合运算引擎
from kousuan.mixed.engine import MixedEngine

from kousuan.core import resolve, ask_ai, update_configuration

def main():
    """Main entry point for the CLI"""
    update_configuration({'llm_model': 'gpt-4o-2024-11-20'})
    if len(sys.argv) == 2 and sys.argv[1].strip() != "calc":
        test_exp(sys.argv[1])
        return
    
    parser = argparse.ArgumentParser(
        description="Kousuan Skill - Mental Arithmetic Calculation Tools",
        prog="kousuan"
    )
    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # Calc skills
    calc_parser = subparsers.add_parser("calc", help="calcuate math problems")
    calc_parser.add_argument("expression", nargs="+", help="expression to calculate e.g., '2+2'")
    calc_parser.add_argument("--all", action="store_true", default=True, help="Output result in JSON format")
    calc_parser.add_argument("--json", action="store_true", help="Output result in JSON format")

    # Ask AI 
    ask_parser = subparsers.add_parser("ask", help="Ask AI for help on math problems")
    ask_parser.add_argument("expression", nargs="+", help="Question to ask AI")
    ask_parser.add_argument("--all", action="store_true", default=True, help="Output result in JSON format")
    ask_parser.add_argument("--json", action="store_true", help="Output result in JSON format")

    # Units command - 新增单位换算命令
    units_parser = subparsers.add_parser("units", help="Unit conversion calculator")
    units_parser.add_argument("expression", nargs="+", help="Unit conversion expression (e.g., '450 秒 =@分')")
    units_parser.add_argument("--json", action="store_true", help="Output result in JSON format")

    # Fraction command - 新增分数计算命令
    fraction_parser = subparsers.add_parser("frac", help="Fraction calculator")
    fraction_parser.add_argument("expression", nargs="+", help="Fraction expression to calculate (e.g., '#frac{1}{2} + #frac{1}{3}')")
    fraction_parser.add_argument("--json", action="store_true", help="Output result in JSON format")

    # Mixed command - 新增混合运算命令
    mixed_parser = subparsers.add_parser("mixed", help="Mixed arithmetic calculator")
    mixed_parser.add_argument("expression", nargs="+", help="Mixed arithmetic expression (e.g., '2+3*4' or '(5+3)*2')")
    mixed_parser.add_argument("--json", action="store_true", help="Output result in JSON format")

    args = parser.parse_args()
    if not args.command and len(sys.argv) == 2:
        test_exp(sys.argv[1])
        return
    elif not args.command:
        parser.print_help()
        return

    try:
        if args.command == "calc":
            handle_calc_command(args)
        elif args.command == "ask":
            handle_ask_command(args)
        elif args.command == "units":
            handle_units_command(args)
        elif args.command == "frac":
            handle_fraction_command(args)
        elif args.command == "mixed":
            handle_mixed_command(args)

    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)

def print_result(result, as_json: bool):
    expression = result.get('problem', '') or result.get('expression', '')
    if as_json:
        valueResult = result.get('result', '')
        result_unit = result.get('result_unit', '')
        if isinstance(valueResult, Fraction):
            ## 使用latex格式输出分数
            result_unit = result.get('latexResult', '')
            valueResult = str(valueResult)
        # JSON格式输出
        output_data = {
            "expression": expression,
            "success": result.get('success'),
            "result": valueResult,
            "name": result.get('name'),
            "result_unit": result_unit,
            "unit_type": result.get('unit_type'),
            "description": result.get('description'),  # 使用新字段名
            "formula": result.get('formula'),  # 添加综合公式字段
            "validation": result.get('validation', False),
        }
        steps = result.get('steps', [])
        if len(steps) > 0 and hasattr(steps[0], 'to_dict'):
            output_data["steps"] = [step.to_dict() for step in steps]
        else:
            output_data["steps"] = steps
        output_data["step_count"] = len(steps)
    
        if not result.get('success'):
            output_data["error"] = result.get('error', 'Unknown error')
        
        print(json.dumps(output_data, ensure_ascii=False, indent=2))
    
    else:
        # 普通格式输出
        if result.get('success'):
            print(f"输入: {expression}")
            print(f"结果: {result['result']} {result.get('result_unit', '')}")
            
            if result.get('name'):
                print(f"算法名称: {result['name']}")
            if result.get('description'):
                print(f"算法描述: {result['description']}")
            if result.get('formula'):
                print(f"换算公式: {result['formula']}")
            if result.get('unit_type'):
                print(f"单位类型: {result['unit_type']}")
            steps = result.get('steps', [])
            if steps:
                print(f"\n计算步骤 (共{len(steps)}步):")
                print("-" * 40)
                for i, step in enumerate(steps, 1):
                    print(f"{i}. {step['description']}")
                    print(f"   操作: {step['operation']}")
                    print(f"   结果: {step['result']}")
                    print(f"   公式: {step.get('formula', '')}")
                    print()
        else:
            print(f"错误: {result.get('error', 'Unknown error')}")
            print(f"输入: {expression}")
            sys.exit(1)

def handle_ask_command(args):
    """处理Ask AI命令"""
    # 将参数列表重新组合为表达式字符串
    expression = " ".join(args.expression)
    try:
        # 求解问题
        results = ask_ai(expression)
        for result in results:
            print_result(result, args.json)
    except Exception as e:
        print(f"处理错误: {e}")
        print(f"输入: {expression}")
        sys.exit(1)

def handle_units_command(args):
    """处理单位换算命令"""
    # 将参数列表重新组合为表达式字符串
    expression = " ".join(args.expression)
    # 创建单位换算引擎
    engine = UnitConversionEngine()
    try:
        # 求解单位换算问题
        result = engine.solve(expression)

        print_result(result, args.json)
    except Exception as e:
        print(f"处理错误: {e}")
        print(f"输入: {expression}")
        sys.exit(1)

def handle_fraction_command(args):
    """处理分数计算命令"""
    # 将参数列表重新组合为表达式字符串
    expression = " ".join(args.expression)
    # 创建分数计算引擎
    engine = FractionEngine()
    try:
        # 求解分数计算问题
        result = engine.solve(expression)
        print_result(result, args.json)
    except Exception as e:
        print(f"处理错误: {e}")
        print(f"输入: {expression}")
        sys.exit(1)

def handle_mixed_command(args):
    """处理混合运算命令"""
    # 将参数列表重新组合为表达式字符串
    expression = " ".join(args.expression)
    # 创建混合运算引擎
    engine = MixedEngine()
    try:
        # 求解混合运算问题
        result = engine.solve(expression)
        print_result(result, args.json)
    except Exception as e:
        print(f"处理错误: {e}")
        print(f"输入: {expression}")
        sys.exit(1)

def handle_calc_command(args):
    """处理计算命令"""
    # 将参数列表重新组合为表达式字符串
    expression = " ".join(args.expression)
    # 创建单位换算引擎
    engine = SmartCalculatorEngine()
    try:
        # 求解单位换算问题
        if args.all:
            results = resolve(expression, optimize=False)
            for result in results:
                print_result(result, args.json)
        else:
            result = engine.solve(expression)
            print_result(result, args.json)
    except Exception as e:
        print(f"处理错误: {e}")
        print(f"输入: {expression}")
        sys.exit(1)

def test_unit():
    # 调试混合运算引擎
    from kousuan.units.unit_engine import UnitConversionEngine
    engine = UnitConversionEngine()
    
    test_expressions = [
        # "20-12/4",   # 应该是17（除法优先）
        # "10-5+5=@",  # 纯加减
        # "10+5-5",    # 纯加减
        # "10+5+5"    # 纯加减
    ]
    
    for expr in test_expressions:
        is_match = engine.is_match_pattern(expr)
        if not is_match:
            print(f"Expression '{expr}' not matches unit conversion pattern.")
            break
        print(f"Testing expression: {expr}")
        result = engine.solve(expr)
        print_result(result, as_json=True)
        print("\n" + "="*50 + "\n")

def test_exp(expr: str, answer : str = ''):
    results = resolve(expr, optimize=True, answer=answer)
    print("\n" + "="*20 +  " 测试开始 " + "="*20 + "\n")
    for result in results:
        print_result(result, as_json=True)
    print("\n" + "="*50 + "\n")

def test():
    import time
    update_configuration({'llm_model': 'gpt-4o-2024-11-20'})
    start_timestamp = time.time()
    test_exp('西瓜每千克3.5元，香蕉每千克4.5元，都买了2千克，问一共多少钱？')
    end_timestamp = time.time()
    print(f"测试耗时: {end_timestamp - start_timestamp:.6f}秒")

if __name__ == "__main__":
    ## 判断参数个数
    if len(sys.argv) < 2:
        print("请提供表达式作为参数")
        test()
        sys.exit(1)




"""
乘以19实现
乘以19，等于原数乘以20再减去原数
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class MultiplyByNineteen(MathCalculator):
    """乘以19算法"""
    
    def __init__(self):
        super().__init__("乘19速算", "乘以19等于乘20再减原数", priority=3)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：任意数乘以19"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, (int, float)) and isinstance(b, (int, float))):
                return False
            
            # 检查是否有一个数是19
            return a == 19 or b == 19
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建乘19速算步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 确定哪个是19，哪个是被乘数
        if a == 19:
            multiplied_num = b
        else:
            multiplied_num = a
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{multiplied_num} × 19 使用乘19速算法",
            operation="识别模式",
            result="19 = 20 - 1，转换为乘20减原数"
        ))
        
        # 计算乘以20
        times_twenty = multiplied_num * 20
        steps.append(CalculationStep(
            description=f"{multiplied_num} × 20 = {times_twenty}",
            operation="乘以20",
            result=times_twenty
        ))
        
        # 减去原数得到最终结果
        final_result = times_twenty - multiplied_num
        steps.append(CalculationStep(
            description=f"{times_twenty} - {multiplied_num} = {final_result}",
            operation="减去原数",
            result=final_result,
            formula="a × 19 = a × 20 - a"
        ))
        
        return steps


"""
基准数法减法实现
找一个与两个数都接近的整十、整百数作为基准，计算它们与基准的差，再求差
适用于两个数都接近同一个基准数的情况
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class ReferenceNumberSubtraction(MathCalculator):
    """基准数法减法算法"""
    
    def __init__(self):
        super().__init__("基准数法减法", "以基准数为参照计算差值，适用于两数都接近某个整数", priority=4)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两个数都接近同一个整十或整百数"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            minuend, subtrahend = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(minuend, (int, float)) and isinstance(subtrahend, (int, float))):
                return False
            
            # 转换为整数便于计算
            minuend, subtrahend = int(minuend), int(subtrahend)
            
            # 只处理正数且被减数大于减数
            if minuend <= subtrahend or minuend < 10 or subtrahend < 10:
                return False
            
            # 寻找合适的基准数
            reference = self._find_best_reference(minuend, subtrahend)
            return reference is not None
            
        except:
            return False
    
    def _find_best_reference(self, minuend: int, subtrahend: int):
        """找到最佳基准数"""
        # 候选基准数
        candidates = []
        
        # 计算数值范围
        min_val = min(minuend, subtrahend)
        max_val = max(minuend, subtrahend)
        
        # 添加整十数候选
        for i in range(max(1, min_val // 10), (max_val // 10) + 2):
            candidate = i * 10
            dist1 = abs(minuend - candidate)
            dist2 = abs(subtrahend - candidate)
            max_dist = max(dist1, dist2)
            if max_dist <= 10:  # 最大距离不超过10
                candidates.append((candidate, max_dist, "十"))
        
        # 添加整百数候选
        if max_val >= 50:
            for i in range(max(1, min_val // 100), (max_val // 100) + 2):
                candidate = i * 100
                dist1 = abs(minuend - candidate)
                dist2 = abs(subtrahend - candidate)
                max_dist = max(dist1, dist2)
                if max_dist <= 20:  # 整百数允许更大的距离
                    candidates.append((candidate, max_dist, "百"))
        
        if not candidates:
            return None
        
        # 选择距离最小的基准数
        best = min(candidates, key=lambda x: x[1])
        return best[0], best[2]
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建基准数法步骤"""
        numbers = formula.get_numbers()
        minuend = int(numbers[0].get_numeric_value())
        subtrahend = int(numbers[1].get_numeric_value())
        
        # 寻找最佳基准数
        result = self._find_best_reference(minuend, subtrahend)
        if result is None:
            raise ValueError("No suitable reference found")
        
        reference, unit_type = result
        
        # 计算与基准数的差值
        minuend_diff = minuend - reference
        subtrahend_diff = subtrahend - reference
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用基准数法",
            operation="识别基准数法",
            result=f"选择整{unit_type}数 {reference} 作为基准数"
        ))
        
        steps.append(CalculationStep(
            description=f"计算与基准数的差值",
            operation="计算差值",
            result=f"{minuend} 比 {reference} {'多' if minuend_diff >= 0 else '少'} {abs(minuend_diff)}；{subtrahend} 比 {reference} {'多' if subtrahend_diff >= 0 else '少'} {abs(subtrahend_diff)}"
        ))
        
        # 用基准数表示原数
        steps.append(CalculationStep(
            description=f"用基准数表示：{minuend} = {reference} + ({minuend_diff:+})，{subtrahend} = {reference} + ({subtrahend_diff:+})",
            operation="基准数表示",
            result=f"转换为基准数形式"
        ))
        
        # 应用减法性质
        steps.append(CalculationStep(
            description=f"应用减法性质：[{reference} + ({minuend_diff:+})] - [{reference} + ({subtrahend_diff:+})] = ({minuend_diff:+}) - ({subtrahend_diff:+})",
            operation="减法性质",
            result="基准数相互抵消"
        ))
        
        # 计算最终结果
        final_result = minuend_diff - subtrahend_diff
        steps.append(CalculationStep(
            description=f"计算差值的差：({minuend_diff:+}) - ({subtrahend_diff:+}) = {final_result}",
            operation="计算结果",
            result=final_result,
            formula=f"基准数法：(a+x) - (a+y) = x - y，其中a={reference}"
        ))
        
        return steps


"""
带余数除法速算算子
匹配带有“······”余数符号的除法题型
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep
import re

class DivisionWithRemainder(MathCalculator):
    def __init__(self):
        super().__init__("带余数除法", "速算带余数的除法题型", priority=2)

    def is_match_pattern(self, formula: Formula) -> bool:
        expr = formula.original_expression
        # 1. 带余数除法题型
        if formula.type == "division" and ("······" in expr or (formula.answer and "······" in str(formula.answer))):
            return True
        # 2. 逆运算最大估值题型（@×8>49/@÷8>49等）
        if "@" in expr and re.search(r'[><=]?', expr):
            return True
        return False

    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        expr = formula.original_expression.replace(' ', '')
        # 1. 逆运算：@÷8=5······4 形式
        match = re.match(r'@÷(\d+)=([\d]+)······([\d]+)', expr)
        if match:
            divisor = int(match.group(1))
            quotient = int(match.group(2))
            remainder = int(match.group(3))
            result = quotient * divisor + remainder
            return [
                CalculationStep(
                    description=f"逆运算：@=商×除数+余数",
                    operation="逆运算",
                    result=result,
                    formula=f"@={quotient}×{divisor}+{remainder}={result}"
                )
            ]
        # 2. 逆运算：@÷9=2······3
        match = re.match(r'@÷(\d+)=([\d]+)······([\d]+)', expr)
        if match:
            divisor = int(match.group(1))
            quotient = int(match.group(2))
            remainder = int(match.group(3))
            result = quotient * divisor + remainder
            return [
                CalculationStep(
                    description=f"逆运算：@=商×除数+余数",
                    operation="逆运算",
                    result=result,
                    formula=f"@={quotient}×{divisor}+{remainder}={result}"
                )
            ]
        # 3. 逆运算：11÷@=3······2 形式
        match = re.match(r'(\d+)÷@=([\d]+)······([\d]+)', expr)
        if match:
            dividend = int(match.group(1))
            quotient = int(match.group(2))
            remainder = int(match.group(3))
            # @ = (dividend - remainder) // quotient
            if quotient == 0:
                return [CalculationStep(description="商为0，无法计算", operation="错误", result="错误")]
            divisor = (dividend - remainder) // quotient
            return [
                CalculationStep(
                    description=f"逆运算：@=(被除数-余数)÷商",
                    operation="逆运算",
                    result=divisor,
                    formula=f"@=({dividend}-{remainder})÷{quotient}={divisor}"
                )
            ]
        def compare_and_compute(factor: int, operator: str, target: int) -> List[CalculationStep]:
            result = target // factor
            remainder = target % factor
            description=f"最大估值：@=目标数÷因数取整"
            formula_text = f"@={target}÷{factor}={result}...{remainder}"
            print(formula_text)
            if remainder == 0:
                formula_text = f"{target}x{factor}={result}"
            if  remainder >= 0 and operator == '>':
                result = int(result) + 1
                formula_text += f";因为余数{remainder}≥0，满足‘>’条件，结果+1"
            elif remainder == 0 and operator == '<':
                result = int(result) - 1
                formula_text += f";因为余数{remainder}=0，满足‘<’条件，结果-1"
            if operator == '=':
                description=f"直接整除"
            
            return [
                CalculationStep(
                    description=description,
                    operation="乘法转除法",
                    result=result,
                    formula=formula_text
                )
            ]
        # 4. 最大估值：@×8>49 或 7×@>18
        match = re.match(r'@×(\d+)([><=]?)(\d+)', expr)
        if not match:
            match = re.match(r'(\d+)×@([><=]?)(\d+)', expr)
        if match:
            factor = int(match.group(1))
            operator = match.group(2)
            target = int(match.group(3))
            result = target // factor
            return compare_and_compute(factor, expr[expr.index(str(target))-1], target)
        # 5. 默认原有带余数除法
        numbers = formula.get_numbers()
        if len(numbers) < 2:
            return []
        dividend = numbers[0].get_numeric_value()
        divisor = numbers[1].get_numeric_value()
        if divisor == 0:
            return [CalculationStep(description="除数为0，无法计算", operation="错误", result="错误")]
        quotient = dividend // divisor
        remainder = dividend % divisor
        steps = [
            CalculationStep(
                description=f"计算整数商：{dividend} ÷ {divisor}",
                operation="取整商",
                result=quotient,
                formula=f"{dividend} ÷ {divisor} = {quotient}"
            ),
            CalculationStep(
                description=f"计算余数：{dividend} % {divisor}",
                operation="取余数",
                result=remainder,
                formula=f"{dividend} % {divisor} = {remainder}"
            ),
            CalculationStep(
                description="组合最终结果",
                operation="格式化为‘商······余数’",
                result=f"{quotient}······{remainder}",
                formula=f"{dividend} ÷ {divisor} = {quotient}······{remainder}"
            )
        ]
        return steps



"""
同数相减实现
处理两个相同数字的减法，结果恒为0
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class SameNumberSubtraction(MathCalculator):
    """同数相减算法"""
    
    def __init__(self):
        super().__init__("同数相减", "两个相同数字的减法，结果恒为0", priority=8)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两个相同的数字相减"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            minuend, subtrahend = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(minuend, int) and isinstance(subtrahend, int)):
                return False
            
            # 两个数必须完全相等
            return minuend == subtrahend
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建同数相减步骤"""
        numbers = formula.get_numbers()
        minuend = int(numbers[0].get_numeric_value())
        subtrahend = int(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 识别同数相减",
            operation="识别同数",
            result="两个相同的数相减"
        ))
        
        steps.append(CalculationStep(
            description=f"任何数减去自身都等于0",
            operation="应用同数相减规律",
            result="结果恒为0"
        ))
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} = 0",
            operation="得出结果",
            result=0,
            formula="a - a = 0"
        ))
        
        return steps


"""
连减变加法实现
将连续减法变为减法和加法的组合
例：126 - 47 - 25 可以变为 126 - (47 + 25) 或调整为更简单的运算
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class ChainSubtractionToAddition(MathCalculator):
    """连减变加法算法"""
    
    def __init__(self):
        super().__init__("连减变加法", "将连续减法转换为加减混合运算", priority=2)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：连续减法或可优化的减法运算"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) < 3:
            return False
        
        try:
            # 检查是否全为整数
            values = [elem.get_numeric_value() for elem in numbers]
            if not all(isinstance(v, (int, float)) for v in values):
                return False
            
            # 至少要有两个数参与运算
            return len(values) >= 2
            
        except:
            return False
    
    def _find_optimization_strategy(self, minuend: float, subtrahends: List[float]) -> dict:
        """寻找最优的连减变加法策略"""
        strategies = []
        
        # 策略1：合并减数后再减
        total_subtrahend = sum(subtrahends)
        if total_subtrahend <= minuend:
            ease_score = self._calculate_ease_score(minuend, total_subtrahend)
            strategies.append({
                'type': 'combine_all',
                'description': f'先计算所有减数之和，再进行减法',
                'ease_score': ease_score,
                'operation': f'{minuend} - ({" + ".join(map(str, subtrahends))})'
            })
        
        # 策略2：寻找凑整机会
        for i, sub1 in enumerate(subtrahends):
            for j, sub2 in enumerate(subtrahends[i+1:], i+1):
                sum_pair = sub1 + sub2
                if sum_pair % 10 == 0 or sum_pair % 100 == 0:
                    remaining = [s for k, s in enumerate(subtrahends) if k != i and k != j]
                    strategies.append({
                        'type': 'pair_round',
                        'description': f'优先计算 {sub1} + {sub2} = {sum_pair}（凑整）',
                        'ease_score': 8,
                        'pair': (sub1, sub2),
                        'remaining': remaining
                    })
        
        # 策略3：寻找接近整十、整百的数进行调整
        for i, subtrahend in enumerate(subtrahends):
            # 检查是否接近整十
            rounded_10 = round(subtrahend / 10) * 10
            diff_10 = abs(subtrahend - rounded_10)
            
            if 1 <= diff_10 <= 3:
                remaining = [s for k, s in enumerate(subtrahends) if k != i]
                if subtrahend < rounded_10:  # 如 97 → 100 - 3
                    strategies.append({
                        'type': 'round_adjust',
                        'description': f'将 {subtrahend} 调整为 {rounded_10} - {diff_10}',
                        'ease_score': 7,
                        'original': subtrahend,
                        'rounded': rounded_10,
                        'adjustment': diff_10,
                        'remaining': remaining,
                        'operation': 'add_back'  # 减数变小了，要加回差值
                    })
                else:  # 如 103 → 100 + 3
                    strategies.append({
                        'type': 'round_adjust',
                        'description': f'将 {subtrahend} 调整为 {rounded_10} + {diff_10}',
                        'ease_score': 7,
                        'original': subtrahend,
                        'rounded': rounded_10,
                        'adjustment': diff_10,
                        'remaining': remaining,
                        'operation': 'subtract_more'  # 减数变大了，要多减
                    })
        
        # 如果没有找到策略，创建一个默认策略
        if not strategies:
            strategies.append({
                'type': 'sequential',
                'description': '按顺序逐步计算',
                'ease_score': 3
            })
        
        # 选择最优策略
        return max(strategies, key=lambda x: x['ease_score'])
    
    def _calculate_ease_score(self, minuend: float, subtrahend: float) -> int:
        """计算运算难易程度评分（1-10，10最容易）"""
        score = 5  # 基础分
        
        # 整十、整百加分
        if subtrahend % 100 == 0:
            score += 3
        elif subtrahend % 10 == 0:
            score += 2
        
        # 结果接近整数加分
        result = minuend - subtrahend
        if result % 100 == 0:
            score += 2
        elif result % 10 == 0:
            score += 1
        
        return min(score, 10)
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建连减变加法步骤"""
        numbers = formula.get_numbers()
        minuend = float(numbers[0].get_numeric_value())
        subtrahends = [float(num.get_numeric_value()) for num in numbers[1:]]
        
        steps = []
        
        original_expression = f"{minuend} - " + " - ".join(map(str, subtrahends))
        steps.append(CalculationStep(
            description=f"原式：{original_expression}",
            operation="识别连减运算",
            result="寻找最优的连减变加法策略"
        ))
        
        # 寻找最优策略
        strategy = self._find_optimization_strategy(minuend, subtrahends)
        
        if strategy:
            if strategy['type'] == 'combine_all':
                # 合并所有减数
                total_subtrahend = sum(subtrahends)
                steps.append(CalculationStep(
                    description=f"合并所有减数：{' + '.join(map(str, subtrahends))} = {total_subtrahend}",
                    operation="合并减数",
                    result=f"原式变为：{minuend} - {total_subtrahend}"
                ))
                
                final_result = minuend - total_subtrahend
                steps.append(CalculationStep(
                    description=f"{minuend} - {total_subtrahend} = {final_result}",
                    operation="执行减法",
                    result=final_result,
                    formula="a - b - c - ... = a - (b + c + ...)"
                ))
                
            elif strategy['type'] == 'pair_round':
                # 优先处理能凑整的一对数
                sub1, sub2 = strategy['pair']
                pair_sum = sub1 + sub2
                remaining = strategy['remaining']
                
                steps.append(CalculationStep(
                    description=f"优先计算凑整的一对：{sub1} + {sub2} = {pair_sum}",
                    operation="凑整配对",
                    result=f"原式变为：{minuend} - {pair_sum}" + (f" - {' - '.join(map(str, remaining))}" if remaining else "")
                ))
                
                intermediate_result = minuend - pair_sum
                if remaining:
                    steps.append(CalculationStep(
                        description=f"{minuend} - {pair_sum} = {intermediate_result}",
                        operation="先减凑整数",
                        result=f"继续计算：{intermediate_result} - {' - '.join(map(str, remaining))}"
                    ))
                    
                    final_result = intermediate_result - sum(remaining)
                    steps.append(CalculationStep(
                        description=f"{intermediate_result} - {sum(remaining)} = {final_result}",
                        operation="完成剩余减法",
                        result=final_result
                    ))
                else:
                    steps.append(CalculationStep(
                        description=f"最终结果：{intermediate_result}",
                        operation="完成计算",
                        result=intermediate_result,
                        formula="a - b - c = a - (b + c)，优先凑整"
                    ))
                
            elif strategy['type'] == 'round_adjust':
                # 调整接近整数的减数
                original = strategy['original']
                rounded = strategy['rounded']
                adjustment = strategy['adjustment']
                remaining = strategy['remaining']
                
                steps.append(CalculationStep(
                    description=f"调整 {original} → {rounded} {'- ' + str(adjustment) if strategy['operation'] == 'add_back' else '+ ' + str(adjustment)}",
                    operation="数值调整",
                    result=f"原式调整为便于计算的形式"
                ))
                
                if strategy['operation'] == 'add_back':
                    # 减数变小了，要加回差值
                    temp_result = minuend - rounded
                    adjusted_result = temp_result + adjustment
                    if remaining:
                        adjusted_result -= sum(remaining)
                    
                    steps.append(CalculationStep(
                        description=f"{minuend} - {rounded} + {adjustment}" + (f" - {sum(remaining)}" if remaining else "") + f" = {adjusted_result}",
                        operation="调整运算",
                        result=adjusted_result,
                        formula=f"a - {original} = a - {rounded} + {adjustment}"
                    ))
                else:
                    # 减数变大了，要多减
                    temp_result = minuend - rounded
                    adjusted_result = temp_result - adjustment
                    if remaining:
                        adjusted_result -= sum(remaining)
                    
                    steps.append(CalculationStep(
                        description=f"{minuend} - {rounded} - {adjustment}" + (f" - {sum(remaining)}" if remaining else "") + f" = {adjusted_result}",
                        operation="调整运算",
                        result=adjusted_result,
                        formula=f"a - {original} = a - {rounded} - {adjustment}"
                    ))
        
        else:
            # 没有特殊策略，直接逐步计算
            current_result = minuend
            for i, subtrahend in enumerate(subtrahends):
                new_result = current_result - subtrahend
                steps.append(CalculationStep(
                    description=f"{current_result} - {subtrahend} = {new_result}",
                    operation=f"第{i+1}次减法",
                    result=new_result
                ))
                current_result = new_result
            
            steps.append(CalculationStep(
                description=f"最终结果：{current_result}",
                operation="完成计算",
                result=current_result,
                formula="逐步减法"
            ))
        
        return steps


"""
同加同减法实现
将被减数和减数同时加上或减去同一个数，使减数变为整十、整百，简化计算
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class SameAddSameSubtract(MathCalculator):
    """同加同减法算法"""
    
    def __init__(self):
        super().__init__("同加同减法", "同时调整被减数和减数，简化减法计算", priority=5)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：减数接近整十、整百，且调整后计算更方便"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            minuend, subtrahend = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(minuend, (int, float)) and isinstance(subtrahend, (int, float))):
                return False
            
            # 转换为整数
            minuend, subtrahend = int(minuend), int(subtrahend)
            
            # 只处理正数且被减数大于减数
            if minuend <= subtrahend or minuend < 10 or subtrahend < 10:
                return False
            
            # 检查是否有合适的调整方案
            return self._find_best_adjustment(minuend, subtrahend) is not None
            
        except:
            return False
    
    def _find_best_adjustment(self, minuend: int, subtrahend: int):
        """寻找最佳调整方案"""
        candidates = []
        
        # 调整到最近的整十数
        nearest_ten = round(subtrahend / 10) * 10
        if nearest_ten != subtrahend:
            adjustment = nearest_ten - subtrahend
            if abs(adjustment) <= 5:  # 调整幅度不超过5
                new_minuend = minuend + adjustment
                # 检查调整后是否更容易计算
                if new_minuend > 0 and self._is_easier_calculation(new_minuend, nearest_ten):
                    candidates.append((nearest_ten, adjustment, "十"))
        
        # 调整到最近的整百数（如果数字较大）
        if subtrahend >= 50:
            nearest_hundred = round(subtrahend / 100) * 100
            if nearest_hundred != subtrahend:
                adjustment = nearest_hundred - subtrahend
                if abs(adjustment) <= 10:  # 整百数允许更大的调整幅度
                    new_minuend = minuend + adjustment
                    if new_minuend > 0 and self._is_easier_calculation(new_minuend, nearest_hundred):
                        candidates.append((nearest_hundred, adjustment, "百"))
        
        if not candidates:
            return None
        
        # 选择调整幅度最小的方案
        best = min(candidates, key=lambda x: abs(x[1]))
        return best
    
    def _is_easier_calculation(self, new_minuend: int, round_subtrahend: int) -> bool:
        """判断调整后的计算是否更容易"""
        # 整十数、整百数的减法通常更容易
        if round_subtrahend % 10 == 0:
            return True
        if round_subtrahend % 100 == 0:
            return True
        return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建同加同减法步骤"""
        numbers = formula.get_numbers()
        minuend = int(numbers[0].get_numeric_value())
        subtrahend = int(numbers[1].get_numeric_value())
        
        # 找到最佳调整方案
        result = self._find_best_adjustment(minuend, subtrahend)
        if result is None:
            raise ValueError("No suitable adjustment found")
        
        target_subtrahend, adjustment, unit_type = result
        new_minuend = minuend + adjustment
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用同加同减法",
            operation="识别同加同减法",
            result=f"将减数调整为整{unit_type}数 {target_subtrahend}"
        ))
        
        if adjustment > 0:
            steps.append(CalculationStep(
                description=f"同时加 {adjustment}：({minuend} + {adjustment}) - ({subtrahend} + {adjustment}) = {new_minuend} - {target_subtrahend}",
                operation="同时增加",
                result=f"转换为 {new_minuend} - {target_subtrahend}"
            ))
        else:
            steps.append(CalculationStep(
                description=f"同时减 {abs(adjustment)}：({minuend} - {abs(adjustment)}) - ({subtrahend} - {abs(adjustment)}) = {new_minuend} - {target_subtrahend}",
                operation="同时减少",
                result=f"转换为 {new_minuend} - {target_subtrahend}"
            ))
        
        steps.append(CalculationStep(
            description=f"计算简化后的减法：{new_minuend} - {target_subtrahend}",
            operation="简化计算",
            result="整十数减法更容易计算"
        ))
        
        # 计算最终结果
        final_result = new_minuend - target_subtrahend
        steps.append(CalculationStep(
            description=f"{new_minuend} - {target_subtrahend} = {final_result}",
            operation="计算结果",
            result=final_result,
            formula="同加同减法：(a±k) - (b±k) = a - b"
        ))
        
        return steps


"""
两位数减法的通用实现
根据数字特征自动选择最优算法：凑整法、破十法、直接减法等
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class TwoDigitSubtraction(MathCalculator):
    """两位数减法通用算法"""
    
    def __init__(self):
        super().__init__("两位数减法", "根据数字特征自动选择最优减法算法", priority=3)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两位数减法"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            minuend, subtrahend = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(minuend, int) and isinstance(subtrahend, int)):
                return False
            
            # 两位数减法：被减数和减数都是两位数，且结果为正
            return (10 <= minuend <= 99 and 10 <= subtrahend <= 99 and minuend >= subtrahend)
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建两位数减法步骤"""
        numbers = formula.get_numbers()
        minuend = int(numbers[0].get_numeric_value())
        subtrahend = int(numbers[1].get_numeric_value())
        
        # 判断使用哪种算法
        if self._is_adjacent_numbers(minuend, subtrahend):
            return self._construct_adjacent_steps(minuend, subtrahend)
        elif self._is_round_number_subtraction(minuend, subtrahend):
            return self._construct_round_number_steps(minuend, subtrahend)
        elif self._is_complement_to_round(subtrahend):
            return self._construct_complement_steps(minuend, subtrahend)
        elif self._needs_borrowing(minuend, subtrahend):
            return self._construct_borrowing_steps(minuend, subtrahend)
        else:
            return self._construct_direct_steps(minuend, subtrahend)
    
    def _is_adjacent_numbers(self, minuend: int, subtrahend: int) -> bool:
        """判断是否为相邻数相减"""
        return abs(minuend - subtrahend) == 1
    
    def _is_round_number_subtraction(self, minuend: int, subtrahend: int) -> bool:
        """判断是否为整十数减法"""
        return minuend % 10 == 0 and subtrahend % 10 == 0
    
    def _is_complement_to_round(self, subtrahend: int) -> bool:
        """判断减数是否接近整十（可用凑整法）"""
        ones = subtrahend % 10
        return ones >= 8 or ones <= 2  # 接近整十的情况
    
    def _needs_borrowing(self, minuend: int, subtrahend: int) -> bool:
        """判断是否需要借位"""
        minuend_ones = minuend % 10
        subtrahend_ones = subtrahend % 10
        return minuend_ones < subtrahend_ones
    
    def _construct_adjacent_steps(self, minuend: int, subtrahend: int) -> List[CalculationStep]:
        """构建相邻数减法步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用相邻数规律",
            operation="识别相邻数",
            result="相邻数相减结果固定为1"
        ))
        
        result = minuend - subtrahend
        steps.append(CalculationStep(
            description=f"相邻数相减：{minuend} - {subtrahend} = {result}",
            operation="直接得出结果",
            result=result,
            formula="相邻数相减 = 1"
        ))
        
        return steps
    
    def _construct_round_number_steps(self, minuend: int, subtrahend: int) -> List[CalculationStep]:
        """构建整十数减法步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用整十数减法",
            operation="识别整十数",
            result="整十数减法：先减十位，再补零"
        ))
        
        minuend_tens = minuend // 10
        subtrahend_tens = subtrahend // 10
        result_tens = minuend_tens - subtrahend_tens
        
        steps.append(CalculationStep(
            description=f"十位相减：{minuend_tens} - {subtrahend_tens} = {result_tens}",
            operation="计算十位差",
            result=result_tens
        ))
        
        result = result_tens * 10
        steps.append(CalculationStep(
            description=f"补零得到结果：{result_tens}0 = {result}",
            operation="补零",
            result=result,
            formula="整十数减法 = (十位相减)×10"
        ))
        
        return steps
    
    def _construct_complement_steps(self, minuend: int, subtrahend: int) -> List[CalculationStep]:
        """构建凑整法步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用凑整法",
            operation="识别凑整机会",
            result="减数接近整十，先凑整再修正"
        ))
        
        # 找到最近的整十数
        round_number = ((subtrahend + 5) // 10) * 10
        difference = round_number - subtrahend
        
        steps.append(CalculationStep(
            description=f"将{subtrahend}凑整到{round_number}，差值为{difference}",
            operation="确定凑整数",
            result=f"凑整到{round_number}"
        ))
        
        temp_result = minuend - round_number
        steps.append(CalculationStep(
            description=f"先算：{minuend} - {round_number} = {temp_result}",
            operation="凑整计算",
            result=temp_result
        ))
        
        final_result = temp_result + difference
        steps.append(CalculationStep(
            description=f"修正结果：{temp_result} + {difference} = {final_result}",
            operation="修正差值",
            result=final_result,
            formula="凑整法：a - b = (a - round_b) + (round_b - b)"
        ))
        
        return steps
    
    def _construct_borrowing_steps(self, minuend: int, subtrahend: int) -> List[CalculationStep]:
        """构建借位减法步骤（破十法）"""
        steps = []
        
        minuend_tens = minuend // 10
        minuend_ones = minuend % 10
        subtrahend_tens = subtrahend // 10
        subtrahend_ones = subtrahend % 10
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用破十法",
            operation="识别借位需求",
            result="个位不够减，需要向十位借1"
        ))
        
        steps.append(CalculationStep(
            description=f"分解：{minuend} = {minuend_tens}0 + {minuend_ones}, {subtrahend} = {subtrahend_tens}0 + {subtrahend_ones}",
            operation="数字分解",
            result=f"被减数十位:{minuend_tens}，个位:{minuend_ones}；减数十位:{subtrahend_tens}，个位:{subtrahend_ones}"
        ))
        
        # 借位过程
        new_tens = minuend_tens - 1
        new_ones = minuend_ones + 10
        
        steps.append(CalculationStep(
            description=f"向十位借1：{minuend_tens}变成{new_tens}，{minuend_ones}变成{new_ones}",
            operation="借位操作",
            result=f"新的被减数：{new_tens}0 + {new_ones} = {new_tens * 10 + new_ones}"
        ))
        
        # 计算结果
        result_tens = new_tens - subtrahend_tens
        result_ones = new_ones - subtrahend_ones
        
        steps.append(CalculationStep(
            description=f"十位相减：{new_tens} - {subtrahend_tens} = {result_tens}",
            operation="计算十位",
            result=result_tens
        ))
        
        steps.append(CalculationStep(
            description=f"个位相减：{new_ones} - {subtrahend_ones} = {result_ones}",
            operation="计算个位",
            result=result_ones
        ))
        
        final_result = result_tens * 10 + result_ones
        steps.append(CalculationStep(
            description=f"组合结果：{result_tens}0 + {result_ones} = {final_result}",
            operation="合并结果",
            result=final_result,
            formula="破十法：借位后分别计算十位和个位"
        ))
        
        return steps
    
    def _construct_direct_steps(self, minuend: int, subtrahend: int) -> List[CalculationStep]:
        """构建直接减法步骤"""
        steps = []
        
        minuend_tens = minuend // 10
        minuend_ones = minuend % 10
        subtrahend_tens = subtrahend // 10
        subtrahend_ones = subtrahend % 10
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用直接减法",
            operation="识别直接减法",
            result="个位够减，直接计算"
        ))
        
        steps.append(CalculationStep(
            description=f"分解：{minuend} = {minuend_tens}0 + {minuend_ones}, {subtrahend} = {subtrahend_tens}0 + {subtrahend_ones}",
            operation="数字分解",
            result=f"十位:{minuend_tens}-{subtrahend_tens}，个位:{minuend_ones}-{subtrahend_ones}"
        ))
        
        result_tens = minuend_tens - subtrahend_tens
        result_ones = minuend_ones - subtrahend_ones
        
        steps.append(CalculationStep(
            description=f"十位相减：{minuend_tens} - {subtrahend_tens} = {result_tens}",
            operation="计算十位",
            result=result_tens
        ))
        
        steps.append(CalculationStep(
            description=f"个位相减：{minuend_ones} - {subtrahend_ones} = {result_ones}",
            operation="计算个位",
            result=result_ones
        ))
        
        final_result = result_tens * 10 + result_ones
        steps.append(CalculationStep(
            description=f"组合结果：{result_tens}0 + {result_ones} = {final_result}",
            operation="合并结果",
            result=final_result,
            formula="直接减法：分别计算十位和个位"
        ))
        
        return steps